!=================================================================
!
! NAME: 
!
!    Output_Values
!
! FUNCTION:
!
!    Outputs data from run
!
! TYPE/LANGUAGE: 
!
!    F90 module
!
! DESCRIPTION:  
!
!    Manages outputing data to files.  Also includes code to generate 
!    a non-opened file units.  
!
!    If NetCDF output, also has store of values for evenutal output
!
! ROUTINES TO BE USED EXTERNALLY:
!
!    Set_Output_File        - Opens output file
!    Close_Output_File      - Closes output file
!    Write_to_Output_File   - Writes ASCII files (DW estimate only) in same 
!                             format as original code from Gary Wick
!    Write_Profiles_ASCII   - Write profile data to ASCII output
!    Get_File_Unit          - Gets new file unit
!    Setup_NetCDF_grid      - Initialized grids for NetCDF output
!    Add_to_NetCDF_grid     - adds to grids to be written to NetCDF output
!
! ROUTINES USED INTERNALLY:
!
!    Write_Profile_ASCII    - Output profiles to ASCII file
!    Write_Output_NetCDF    - Writes to NetCDF output
!
! HISTORY:
! 
!   Original version by Jonathan P.D. Mittaz on 9/10/2013
!
!=================================================================
MODULE Output_Values

  USE Kinds
  USE Types
  USE ErrorHandler
  USE Constants

  USE NETCDF

  IMPLICIT NONE

  !
  ! Global variables
  !
  ! Output file unit only used internally to this module
  !
  INTEGER(Int4) :: Output_Unit
  INTEGER(Int4) :: Output_Profile_Unit

  !
  ! Store for NetCDF output 
  !
  INTEGER(Int4) :: ntimes
  INTEGER(Int4), ALLOCATABLE :: Store_Time(:)
  REAL(Int4), ALLOCATABLE :: Store_Year(:)
  REAL(RealType), ALLOCATABLE :: Store_DayNo(:)
  REAL(RealType), ALLOCATABLE :: Store_grid_lat(:,:)
  REAL(RealType), ALLOCATABLE :: Store_grid_lon(:,:)
  REAL(RealType), ALLOCATABLE :: Store_DW(:,:,:)
  
  PRIVATE
  PUBLIC :: Set_Output_File
  PUBLIC :: Close_Output_File
  PUBLIC :: Write_to_Output_File
  PUBLIC :: Get_File_Unit
  PUBLIC :: Setup_NetCDF_grid
  PUBLIC :: Free_NetCDF_Store
  PUBLIC :: isLeap

CONTAINS

  !=================================================================
  !
  ! NAME:
  !
  !      Set_Output_File
  !
  ! FUNCTION:
  !
  !      Opens output file
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Opens output file and stores the file unit
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  !      Get_File_Unit
  !      out_ERROR
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Set_Output_File( params, profile_ascii)
  !
  ! INPUTS:
  ! 
  !      params        - Parameter structure
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !
  SUBROUTINE Set_Output_File( params )

    TYPE(Parameter_Type), INTENT(INOUT) :: params

    ! Local variables
    INTEGER(Int4) :: IOS
    
    IF( params%output_ascii_profiles )THEN
       Output_Profile_Unit = Get_File_Unit()
          
       OPEN(UNIT=Output_Profile_Unit,FILE=params%output_file_profile,&
            STATUS='UNKNOWN',IOSTAT=IOS)
       IF( 0 .ne. IOS )THEN
          CALL out_ERROR('Cannot open Profile Output file','Set_Output_File',&
               'Output_Values.f90')
       ENDIF
    ENDIF
    
    IF( params%Output_File_NetCDF_Format )THEN
       !
       ! Setup NetCDF grid storage
       !
       CALL Setup_NetCDF_grid( params )
    ELSE
       Output_Unit = Get_File_Unit()
       
       OPEN(UNIT=Output_Unit,FILE=params%output_file,STATUS='UNKNOWN',&
            IOSTAT=IOS)
       IF( 0 .ne. IOS )THEN
          CALL out_ERROR('Cannot open Output file','Set_Output_File',&
               'Output_Values.f90')
       ENDIF
    ENDIF

  END SUBROUTINE Set_Output_File

  !=================================================================
  !
  ! NAME:
  !
  !      Close_Output_File
  !
  ! FUNCTION:
  !
  !      Closes output file
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Closes output file 
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Close_Output_File( params )
  !
  ! INPUTS:
  ! 
  !      params    - Parameter structure
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !
  SUBROUTINE Close_Output_File( params )

    TYPE(Parameter_Type), INTENT(IN) :: params

    !
    ! Close profile file if opened
    IF( params%output_ascii_profiles )THEN
       CLOSE(Output_Profile_Unit)
    ENDIF
    
    !
    ! If NetCDF output - here iw where we write it out
    !
    IF( params%Output_File_NetCDF_Format )THEN
       CALL Write_NetCDF_Output( params )
    ELSE
       CLOSE(Output_Unit)
    ENDIF

  END SUBROUTINE Close_Output_File

  !=================================================================
  !
  ! NAME:
  !
  !      Write_to_Output_File
  !
  ! FUNCTION:
  !
  !      Writes diurnal warming estimate to ASCII file or stores result
  !      to NetCDF storage
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Writes out dirunal warming estimate (T(1)-T(5m)) to ASCII format
  !      with the same format as the original code or store value ready
  !      to be written to a NetCDF output file
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Write_to_Output_File( iTime, params, surface, profile, 
  !                                 Header )
  !
  ! INPUTS:
  ! 
  !      iTime     - Time step from time loop in main code
  !      params    - Input parameters
  !      surface   - Surface and time data
  !      profile   - Profile data
  !      Header    - Logical to enabel writing header information
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !
  SUBROUTINE Write_to_Output_File( iTime, params, surface, &
       profile, Header )

    INTEGER(Int4), INTENT(IN) :: iTime
    TYPE(parameter_type), INTENT(IN) :: params
    TYPE(Surface_Type), INTENT(IN) :: surface
    TYPE(Profile_Type), INTENT(IN) :: profile
    LOGICAL, INTENT(IN) :: Header

    IF( params%Output_File_NetCDF_Format )THEN
       !
       ! Add DW estimate to the model grid 
       !
       IF( params%use_skin_temperature )THEN
          CALL Add_to_NetCDF_grid( params, iTime, surface%time, surface%Year, &
               surface%i_out, surface%j_out, &
               surface%tskin-profile%temp(profile%pos_reference) )
       ELSE
          CALL Add_to_NetCDF_grid( params, iTime, surface%time, surface%Year, &
               surface%i_out, surface%j_out, &
               profile%temp(1)-profile%temp(profile%pos_reference) )
       ENDIF
    ELSE
       IF( Header )THEN
          WRITE(Output_Unit,'(I8,1x,3(F10.4,1x),2(I4,1x))')&
               surface%date,surface%latitude,surface%longitude,&
               surface%sstinit,surface%i_out,surface%j_out
       ELSE
          IF( iTime .ne. 1 .and. &
               MOD((iTime-1),params%print_freq_loop) .eq. 0 )THEN
             !
             ! test_output is Gary Wicks reduced output format
             !
             IF( params%test_output )THEN
                write(Output_Unit,'(F13.6,1x,7(F10.4,1x))')surface%time, &
                     profile%temp(1), surface%tskin, profile%temp(70), &
                     profile%temp(30), profile%temp(115), surface%qsol, &
                     surface%wind_speed
             ELSE
                !
                ! Just output the DW estimate
                !
                IF( params%use_skin_temperature )THEN
                   !
                   ! Use Tskin
                   !
                   WRITE(Output_Unit,'(F13.6,1x,F10.4)')surface%time,&
                        surface%tskin-profile%temp(profile%pos_reference)
                ELSE
                   !
                   ! Use Tsubskin
                   !
                   WRITE(Output_Unit,'(F13.6,1x,F10.4)')surface%time,&
                        profile%temp(1)-profile%temp(profile%pos_reference)
                ENDIF
             ENDIF
          ENDIF
       ENDIF
    ENDIF

    !
    ! If writing out ASCII profiles
    IF( params%output_ascii_profiles .and. .not.Header )THEN
       CALL Write_Profile_ASCII(surface, profile)
    ENDIF

  END SUBROUTINE Write_to_Output_File

  !=================================================================
  !
  ! NAME:
  !
  !      Write_Profile_ASCII
  !
  ! FUNCTION:
  !
  !      Writes profile data to ASCII file
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Writes out profiles to an ASCII file
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Write_Profile_ASCII( surface, profile )
  !
  ! INPUTS:
  ! 
  !      surface   - Surface and time data
  !      profile   - Profile data
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !
  SUBROUTINE Write_Profile_ASCII( surface, profile )

    TYPE(Surface_Type), INTENT(IN) :: surface
    TYPE(Profile_Type), INTENT(IN) :: profile

    ! Local variables
    
    WRITE(Output_Profile_Unit,*)surface%time
    WRITE(Output_Profile_Unit,*)profile%nlevels
    WRITE(Output_Profile_Unit,*)surface%wind_speed,surface%qsol
    WRITE(Output_Profile_Unit,*)profile%z*profile%bottom
    WRITE(Output_Profile_Unit,*)profile%swrad
    WRITE(Output_Profile_Unit,*)profile%dm_temp
    WRITE(Output_Profile_Unit,*)profile%temp
    WRITE(Output_Profile_Unit,*)profile%salinity
    WRITE(Output_Profile_Unit,*)profile%Velocity_U
    WRITE(Output_Profile_Unit,*)profile%Velocity_V
    WRITE(Output_Profile_Unit,*)profile%Velocity_U_Stokes
    WRITE(Output_Profile_Unit,*)profile%Velocity_V_Stokes
    WRITE(Output_Profile_Unit,*)profile%Q2
    WRITE(Output_Profile_Unit,*)profile%Q2L
    WRITE(Output_Profile_Unit,*)profile%rho
    WRITE(Output_Profile_Unit,*)profile%richardson_no
    WRITE(Output_Profile_Unit,*)profile%shear_prod
    WRITE(Output_Profile_Unit,*)profile%buoyancy_prod
    WRITE(Output_Profile_Unit,*)profile%shear_prod_stokes
    WRITE(Output_Profile_Unit,*)profile%prod
    WRITE(Output_Profile_Unit,*)profile%KQ
    WRITE(Output_Profile_Unit,*)profile%KN
    WRITE(Output_Profile_Unit,*)profile%KM
    WRITE(Output_Profile_Unit,*)profile%KH

  END SUBROUTINE Write_Profile_ASCII

  !=================================================================
  !
  ! NAME:
  !
  !      Get_File_Unit
  !
  ! FUNCTION:
  !
  !      Gets an available file unit
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Get the next available file unit.  Ensures file units are available
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      unit = Get_File_Unit()
  !
  ! INPUTS:
  !
  ! OUTPUTS:
  !
  ! RETURNS:
  !
  !       new file unit
  !
  ! HISTORY
  !
  !      Based on code supplied by Gary Wick NOAA/OAR
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !      
  !
  INTEGER(Int4) FUNCTION Get_File_Unit()

    ! Local variables
    INTEGER(Int4) :: I
    LOGICAL :: open_logical

    Get_File_Unit = -1
    unit_loop: DO I=20,200
       INQUIRE(Unit=I,OPENED=open_logical)
       IF( .not. open_logical )THEN
          Get_File_Unit = I
          EXIT unit_loop
       ENDIF
    END DO unit_loop

    IF( -1 .eq. Get_File_Unit )THEN
       CALL out_ERROR('Ran out of file units','Get_File_Unit','get_data.f90')
    ENDIF

  END FUNCTION Get_File_Unit

  !=================================================================
  !
  ! NAME:
  !
  !      Setup_NetCDF_grid
  !
  ! FUNCTION:
  !
  !      Sets up grids to store output to be written to NetCDF file
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Allocated and initilizes grids which will be written out as 
  !      the NetCDF output
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Setup_NetCDF_grid( params )
  !
  ! INPUTS:
  !
  !      params - input parameters
  !
  ! OUTPUTS:
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Original F90 version by Jonathan P.D. Mittaz 11/21/2013
  !      
  !
  SUBROUTINE Write_NetCDF_Output( params )

    TYPE(Parameter_Type), INTENT(IN) :: params

    ! Local variables
    INTEGER(Int4) :: status
    INTEGER(Int4) :: nc_id

    INTEGER(Int4) :: dim_nx
    INTEGER(Int4) :: dim_ny
    INTEGER(Int4) :: dim_time
    
    INTEGER(Int4) :: dimids_1dim(1)
    INTEGER(Int4) :: dimids_2dim(2)
    INTEGER(Int4) :: dimids_3dim(3)
    INTEGER(Int4) :: varid_time
    INTEGER(Int4) :: varid_year
    INTEGER(Int4) :: varid_dayno
    INTEGER(Int4) :: varid_lon
    INTEGER(Int4) :: varid_lat
    INTEGER(Int4) :: varid_dw

    !
    ! Open/Create NetCDF Output File
    !
    status = nf90_create(path=params%output_file, cmode=NF90_CLOBBER, &
         ncid=nc_id)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Open/Creating NetCDF output file','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    !
    ! Set Global attributes
    !
    status = nf90_put_att(ncid=nc_id, varid=NF90_GLOBAL, name='Version', &
         values=Code_Version)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr Version','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    status = nf90_put_att(ncid=nc_id, varid=NF90_GLOBAL, name='minLat', &
         values=params%grid_min_lat)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr minLat','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    status = nf90_put_att(ncid=nc_id, varid=NF90_GLOBAL, name='latStep', &
         values=params%lat_step)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr latStep','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    status = nf90_put_att(ncid=nc_id, varid=NF90_GLOBAL, name='minLon', &
         values=params%grid_min_lon)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr minLon','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    status = nf90_put_att(ncid=nc_id, varid=NF90_GLOBAL, name='lonStep', &
         values=params%lon_step)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr lonStep','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    status = nf90_put_att(ncid=nc_id, varid=NF90_GLOBAL, name='Year', &
         values=params%year)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr Year','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    status = nf90_put_att(ncid=nc_id, varid=NF90_GLOBAL, name='Month', &
         values=params%month)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr Month','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    status = nf90_put_att(ncid=nc_id, varid=NF90_GLOBAL, name='Day', &
         values=params%day)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr Day','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    !
    ! Setup dimensions
    !
    status = nf90_def_dim(ncid=nc_id, name='lon', len=params%grid_nx, &
         dimid=dim_nx)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining X dimension (lon)','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    status = nf90_def_dim(ncid=nc_id, name='lat', len=params%grid_ny, &
         dimid=dim_ny)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Y dimension (lat)','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    status = nf90_def_dim(ncid=nc_id, name='time', len=ntimes, &
         dimid=dim_time)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Time dimension','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    !
    ! Define variables
    !
    ! For Time (1 dimension)
    !
    dimids_1dim(1) = dim_time
    status = nf90_def_var(ncid=nc_id,name='time',xtype=NF90_INT,&
         dimids=dimids_1dim,varid=varid_time)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining time variable','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF
    
    status = nf90_put_att(ncid=nc_id, varid=varid_time, name='COMMENT', &
         values='Number of seconds from 01-01-1981 (GHRSST time)')
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr dayno','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    dimids_1dim(1) = dim_time
    status = nf90_def_var(ncid=nc_id,name='Year',xtype=NF90_INT,&
         dimids=dimids_1dim,varid=varid_year)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining time variable','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF
    
    status = nf90_put_att(ncid=nc_id, varid=varid_year, name='COMMENT', &
         values='Year for each time slot')
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr dayno','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    dimids_1dim(1) = dim_time
    status = nf90_def_var(ncid=nc_id,name='DayNo',xtype=NF90_FLOAT,&
         dimids=dimids_1dim,varid=varid_dayno)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining DayNo variable','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF
    
    status = nf90_put_att(ncid=nc_id, varid=varid_dayno, name='COMMENT', &
         values='Fractional day number for each time slot')
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr dayno','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    !
    ! For longitude/latitude - 2dim variables
    !
    dimids_2dim(1) = dim_nx
    dimids_2dim(2) = dim_ny

    !
    ! Define longitude/latitude
    !
    status = nf90_def_var(ncid=nc_id,name='longitude',xtype=NF90_FLOAT,&
         dimids=dimids_2dim,varid=varid_lon)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining longitude variable','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF
         
    status = nf90_put_att(ncid=nc_id, varid=varid_lon, name='COMMENT', &
         values='Longitude (E+ve, 0 - 360 degrees) on model grid')
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr Longitude','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    status = nf90_def_var(ncid=nc_id,name='latitude',xtype=NF90_FLOAT,&
         dimids=dimids_2dim,varid=varid_lat)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining latitude variable','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    status = nf90_put_att(ncid=nc_id, varid=varid_lat, name='COMMENT', &
         values='Latitude (N+ve, -90 - 90 degrees) on model grid')
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr Latitude','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    !
    ! Define 3 dim variable (DWestimate)
    !
    dimids_3dim(1) = dim_nx
    dimids_3dim(2) = dim_ny
    dimids_3dim(3) = dim_time

    !
    ! Define DWestimate
    !
    status = nf90_def_var(ncid=nc_id,name='DW',xtype=NF90_FLOAT,&
         dimids=dimids_3dim,varid=varid_dw)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining DW variable','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    IF( params%use_skin_temperature )THEN
       status = nf90_put_att(ncid=nc_id, varid=varid_dw, name='COMMENT', &
            values='Diurnal warming estimate (Tskin - T(5m))')
    ELSE
       status = nf90_put_att(ncid=nc_id, varid=varid_dw, name='COMMENT', &
            values='Diurnal warming estimate (T(1) - T(5m))')
    ENDIF
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr DW','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    status = nf90_put_att(ncid=nc_id, varid=varid_dw, name='_FillValue', &
         values=-1e30)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Defining Attr DW(fill)','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

    !
    ! Close define mode
    !
    status = nf90_enddef(ncid=nc_id)

    !
    ! Write to variables
    !
    ! Time
    ! 
    status = nf90_put_var(ncid=nc_id,varid=varid_time,values=Store_Time)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Cannot write time to NetCDF output file',&
            'Write_NetCDF_Output','Output_Values.f90')
    ENDIF

    status = nf90_put_var(ncid=nc_id,varid=varid_year,values=Store_Year)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Cannot write Year to NetCDF output file',&
            'Write_NetCDF_Output','Output_Values.f90')
    ENDIF

    status = nf90_put_var(ncid=nc_id,varid=varid_dayno,values=Store_DayNo)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Cannot write DayNo to NetCDF output file',&
            'Write_NetCDF_Output','Output_Values.f90')
    ENDIF

    !
    ! Longitude/Latitude
    ! 
    status = nf90_put_var(ncid=nc_id,varid=varid_lon,values=Store_grid_lon)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Cannot write longitude to NetCDF output file',&
            'Write_NetCDF_Output','Output_Values.f90')
    ENDIF

    status = nf90_put_var(ncid=nc_id,varid=varid_lat,values=Store_grid_lat)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Cannot write latitude to NetCDF output file',&
            'Write_NetCDF_Output','Output_Values.f90')
    ENDIF
    
    !
    ! DW estimate
    !
    status = nf90_put_var(ncid=nc_id,varid=varid_dw,values=Store_DW)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Cannot write longitude to NetCDF output file',&
            'Write_NetCDF_Output','Output_Values.f90')
    ENDIF

    !
    ! Close NetCDF file
    !
    status = nf90_close(ncid=nc_id)
    IF( nf90_noerr .ne. status )THEN
       CALL out_ERROR('Closing NetCDF output file','Write_NetCDF_Output',&
            'Output_Values.f90')
    ENDIF

  END SUBROUTINE Write_NetCDF_Output

  !=================================================================
  !
  ! NAME:
  !
  !      Setup_NetCDF_grid
  !
  ! FUNCTION:
  !
  !      Sets up grids to store output to be written to NetCDF file
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Allocated and initilizes grids which will be written out as 
  !      the NetCDF output
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Setup_NetCDF_grid( params )
  !
  ! INPUTS:
  !
  !      params - input parameters
  !
  ! OUTPUTS:
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Original F90 version by Jonathan P.D. Mittaz 11/21/2013
  !      
  !
  SUBROUTINE Setup_NetCDF_grid( params )

    TYPE(Parameter_Type), INTENT(INOUT) :: params    

    ! Local variables
    INTEGER(Int4) :: I
    INTEGER(Int4) :: J
    INTEGER(Int4) :: STAT

    !
    ! Get number of entries based on length of run and output step
    !
    ntimes = params%days*24._RealType/params%print_freq_hours
    
    !
    ! Allocate output arrays
    !
    ALLOCATE(Store_DW(params%grid_nx,params%grid_ny,ntimes),&
         Store_grid_lat(params%grid_nx,params%grid_ny),&
         Store_grid_lon(params%grid_nx,params%grid_ny),&
         Store_Time(ntimes),Store_Year(ntimes),Store_DayNo(ntimes),STAT=STAT)
    IF( 0 .ne. STAT )THEN
       CALL out_ERROR('Allocating DW Store arrays','Setup_NetCDF_grid',&
            'Output_Values.f90')
    ENDIF

    params%lon_step = (params%grid_max_lon - params%grid_min_lon)/&
         (params%grid_nx-1)
    params%lat_step = (params%grid_max_lat - params%grid_min_lat)/&
         (params%grid_ny-1)

    !
    ! Fill latitude and longitude arrays
    !
    DO I=1,params%grid_ny
       DO J=1,params%grid_nx
          Store_grid_lat(J,I) = params%grid_min_lat + (I-1)*params%lat_step
          Store_grid_lon(J,I) = params%grid_min_lon + (J-1)*params%lon_step
       END DO
    END DO

    !
    ! Set DW value to BAD (-1e30)
    !
    Store_DW = -1e30

  END SUBROUTINE Setup_NetCDF_grid

  !=================================================================
  !
  ! NAME:
  !
  !      Add_To_NetCDF_grid
  !
  ! FUNCTION:
  !
  !      Adds DW estimate to NetCDF output grid
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Adds DW estiamte to grid that will be written out as a NetCDF
  !      file
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Add_To_NetCDF_grid( params, iTime, time, year, ipos, jpos, 
  !                               DWestimate )
  !
  ! INPUTS:
  !
  !      params     - input parameters
  !      iTime      - input time loop step
  !      time       - actual time (fractional day number)
  !      year       - year
  !      ipos       - X location in model grid
  !      jpos       - Y location in model grid
  !      DWestimate - estimate of DW
  !
  ! OUTPUTS:
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Original F90 version by Jonathan P.D. Mittaz 11/21/2013
  !      
  !
  SUBROUTINE Add_to_NetCDF_grid( params, iTime, time, year, &
       ipos, jpos, DWestimate )

    TYPE(Parameter_Type), INTENT(IN) :: params
    INTEGER(Int4), INTENT(IN) :: iTime
    REAL(Dble), INTENT(IN) :: time
    INTEGER(Int4), INTENT(IN) :: year
    INTEGER(Int4), INTENT(IN) :: ipos
    INTEGER(Int4), INTENT(IN) :: jpos
    REAL(RealType), INTENT(IN) :: DWestimate

    ! Local variables
    INTEGER(Int4) :: time_loc

    !
    ! Make sure that time is right to be added in
    !
    IF( MOD((iTime-1),params%print_freq_loop) .eq. 0 )THEN    
       !
       ! Get time position in output array
       !
       time_loc = ((iTime-1)/params%print_freq_loop) + 1

       !
       ! Check locations 
       !
       IF( (time_loc .le. 0 .or. time_loc .gt. ntimes) .or. &
            (ipos .le. 0 .or. ipos .gt. params%grid_nx) .or. &
            (jpos .le. 0 .or. jpos .gt. params%grid_ny) )THEN
          print *,time_loc,ipos,jpos
          CALL out_ERROR('time/xpos/ypos out of range','Add_to_NetCDF_grid',&
               'Output_Values.f90')
       ENDIF

       !
       ! Store values
       !
       ! Note Store_Time is number of seconds from 01-01-1981 (GHRSST Time)
       !
       Store_Time(time_loc) = get_GHRSST_Time(year,time)
       Store_Year(time_loc) = year
       Store_DayNo(time_loc) = time
       Store_DW(ipos,jpos,time_loc) = DWestimate

    ENDIF

  END SUBROUTINE Add_to_NetCDF_grid

  !=================================================================
  !
  ! NAME:
  !
  !      Free_NetCDF_Store
  !
  ! FUNCTION:
  !
  !      Frees arrays used to store output to be written to NetCDF file
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Frees arrays used to store output to be written to NetCDF file
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Free_NetCDF_Store( params )
  !
  ! INPUTS:
  !
  !      params - input parameters
  !
  ! OUTPUTS:
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Original F90 version by Jonathan P.D. Mittaz 12/03/2013
  !      
  SUBROUTINE Free_NetCDF_Store(params)

    TYPE(Parameter_Type), INTENT(IN) :: params

    ! Local variables
    INTEGER :: STAT

    IF( params%Output_File_NetCDF_Format )THEN
       DEALLOCATE(Store_time,Store_Year,Store_DayNo,Store_DW,STAT=STAT)
       IF( 0 .ne. STAT )THEN
          CALL out_ERROR('Cannot deallocate store arrays','Free_NetCDF_Store',&
               'Output_Values.f90')
       ENDIF
    ENDIF

  END SUBROUTINE Free_NetCDF_Store

  !=================================================================
  !
  ! NAME:
  !
  !      Get_GHRSST_Time
  !
  ! FUNCTION:
  !
  !      Returns the GHRSST time (seconds from 1981-01-01)
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Returns the GHRSST time (seconds from 1981-01-01)
  !      Uses the same algorithm as is used in the POES-GOES Blended
  !      ingestor for consistency
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      value = Get_GHRSST_Time( Year, DayNo )
  !
  ! INPUTS:
  !
  !      Year   - year
  !      DayNo  - Fraction Day Number (e.g. 79.25)
  !
  ! OUTPUTS:
  !
  ! RETURNS:
  !
  !      value  - GHRSST Time (seconds from 01-01-1981)
  !
  ! HISTORY
  !
  !      Original F90 version by Jonathan P.D. Mittaz 12/03/2013
  !      
  INTEGER(Int4) FUNCTION Get_GHRSST_Time( Year, DayNo )RESULT(Time)

    INTEGER(Int4), INTENT(IN) :: Year
    REAL(Dble), INTENT(IN) :: DayNo

    ! Local variables
    REAL(Dble), PARAMETER :: START_JULDAY = 2444606.5_Dble
    INTEGER(Int4) :: YYYY
    INTEGER(Int4) :: int_DayNo
    INTEGER(Int4) :: Month
    INTEGER(Int4) :: Day
    REAL(Dble) :: Hours
    REAL(Dble) :: julDay 

    !
    ! Make sure Year is correct range 
    !
    IF( Year .lt. 100 )THEN
       IF( Year .lt. 50 )THEN
          YYYY = 2000+Year
       ELSE
          YYYY = 1900+Year
       ENDIF
    ELSE
       YYYY = Year
    ENDIF

    !
    ! Get Dayno and hours from fraction DayNo
    !
    int_DayNo = INT(DayNo)
    Hours = (DayNo - int_DayNo)*24._Dble

    !
    ! Get month day from day number 
    !
    CALL Date_from_Daynum(YYYY,int_Dayno,Month,Day)
    
    !
    ! Get julian day
    !
    julDay = get_Julian_Day( YYYY, Month, Day ) - 0.5_RealType
    
    !
    ! Convert to seconds from 01-01-1981 
    !
    Time = INT((julDay - START_JULDAY)*86400._Dble + Hours*3600._Dble)
    
    RETURN

  END FUNCTION Get_GHRSST_Time

  !=================================================================
  !
  ! NAME:
  !
  !      isLeap
  !
  ! FUNCTION:
  !
  !      Returns .TRUE. if year is a leap year
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Returns .TRUE. if the input year is a leap year
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      value = isLeap( Year )
  !
  ! INPUTS:
  !
  !      Year   - Year
  !
  ! OUTPUTS:
  !
  ! RETURNS:
  !
  !      value  - .TRUE. if a leap year, .FALSE. if not
  !
  ! HISTORY
  !
  !      Original F90 version by Jonathan P.D. Mittaz 02/03/2014
  !      
  LOGICAL FUNCTION isLeap( Year )
    
    INTEGER, INTENT(IN) :: Year

    IF( (0 .eq. MOD(Year,4) .and. 0 .ne. MOD(Year,100)) .or. &
         (0 .eq. MOD(Year,400)) )THEN
       isLeap = .TRUE.
    ELSE
       isLeap = .FALSE.
    ENDIF
    RETURN

  END FUNCTION isLeap

  !=================================================================
  !
  ! NAME:
  !
  !      Date_from_Daynum
  !
  ! FUNCTION:
  !
  !      Returns the Month/Day from day number
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Returns the Month/Day from day number
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Date_from_Daynum( Year, DayNo, Month, Day )
  !
  ! INPUTS:
  !
  !      Year   - Year
  !      DayNo  - Day Number
  !
  ! OUTPUTS:
  !
  !      Month  - Month
  !      Day    - Day
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Original F90 version by Jonathan P.D. Mittaz 12/03/2013
  !      
  SUBROUTINE Date_from_Daynum( Year, Daynum, Month, Day )
    
    INTEGER(Int4), INTENT(IN) :: Year
    INTEGER(Int4), INTENT(IN) :: Daynum
    INTEGER(Int4), INTENT(OUT) :: Month
    INTEGER(Int4), INTENT(OUT) :: Day

    ! Local variables
    INTEGER(Int4) :: I

    INTEGER(Int4), PARAMETER :: N_MONTHS = 13
    INTEGER(Int4), PARAMETER :: dayNumber(N_MONTHS) = &
         (/0,31,59,90,120,151,181,212,243,273,304,334,365/)
    INTEGER(Int4), PARAMETER :: dayNumberLeap(N_MONTHS) = &
         (/0,31,60,91,121,152,182,213,244,274,305,335,366/)

    ! If year is leap year use leap year day numbers in year 
    IF( isLeap(Year) )THEN
       ! Loop round number of months to find correct month 
       DO I=2,N_MONTHS
          IF( daynum .le. dayNumberLeap(i) )THEN
             ! Get month and day and return
             Month = i-1
             Day = Daynum - dayNumberLeap(i-1)
             RETURN
          ENDIF
       END DO
    ELSE 
       ! Not a leap year 
       ! Loop round number of months to find correct month 
       DO I=2,N_MONTHS
          IF( daynum .le. dayNumber(i) )THEN
             ! Get month and day and return 
             Month = i-1
             Day = Daynum - dayNumber(i-1)
             RETURN
          ENDIF
       END DO
    ENDIf
    
    !
    ! Shouldn't get here
    !
    CALL out_ERROR('Cant get Date_from_Daynum','Date_from_Daynum',&
         'Output_Values.f90')

  END SUBROUTINE Date_from_Daynum

  !=================================================================
  !
  ! NAME:
  !
  !      Get_Julian_Day
  !
  ! FUNCTION:
  !
  !      Returns the julian day
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Returns the julian day from year, month, day
  !      Based on code from the POES-GOES Ingestor
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      jday = Get_Julian_Day( Year, Month, Day )
  !
  ! INPUTS:
  !
  !      Year   - Year
  !      Month  - Month
  !      Day    - Day
  !
  ! OUTPUTS:
  !
  ! RETURNS:
  !
  !      jday  - Julian day 
  !
  ! HISTORY
  !
  !      Original F90 version by Jonathan P.D. Mittaz 12/03/2013
  !      
  INTEGER(Int4) FUNCTION Get_Julian_Day( Year, Month, Day )RESULT(julday)

    INTEGER(Int4), INTENT(IN) :: Year
    INTEGER(Int4), INTENT(IN) :: Month
    INTEGER(Int4), INTENT(IN) :: Day

    ! Local variables
    INTEGER(Int4), PARAMETER :: IGREG = (15+31*(10+12*1582))
    INTEGER(Int4) :: jy
    INTEGER(Int4) :: jm
    INTEGER(Int4) :: ja

    jy = Year
    IF( Month .gt. 2 )THEN
       jm = Month+1
    ELSE
       jy = jy-1
       jm = Month+13
    ENDIF

    julday = INT(365.25*jy) + INT(30.6001*jm) + Day + 1720995

    IF( Day+31*(Month+12*Year) .ge. IGREG )THEN
       ja = INT(0.01*jy)
       julday = julday + 2 - ja + INT(0.25*ja)
    ENDIF

    RETURN

  END FUNCTION Get_Julian_Day

END MODULE Output_Values
