!=================================================================
!
! NAME: 
!
!    Types
!
! FUNCTION:
!
!    Sets up the types (structures) to be used in the code
!
! TYPE/LANGUAGE: 
!
!    F90 module
!
! DESCRIPTION:  
!
!    Contains the defined types (structires) used in the Diurnal Warming 
!    (Kantha Clayson) code
!
! ROUTINES TO BE USED EXTERNALLY:
!
! ROUTINES USED INTERNALLY:
!
! HISTORY:
! 
!   Original version by Jonathan P.D. Mittaz on 9/18/2013
!
!=================================================================
MODULE Types

  USE Kinds

  IMPLICIT NONE

  !
  ! Turbulence constants structure
  !
  TYPE Turb_Cnts_Type
     REAL(RealType) :: A1
     REAL(RealType) :: B1
     REAL(RealType) :: A2
     REAL(RealType) :: B2
     REAL(RealType) :: C1
     REAL(RealType) :: C2
     REAL(RealType) :: C3
     REAL(RealType) :: E1
     REAL(RealType) :: E2
     REAL(RealType) :: E3
     REAL(RealType) :: E4
     REAL(RealType) :: E5
     REAL(RealType) :: E6
     REAL(RealType) :: slsq
     REAL(RealType) :: CF1
     REAL(RealType) :: CF2
     REAL(RealType) :: CF3
     REAL(RealType) :: CF4
     REAL(RealType) :: CF5
     REAL(RealType) :: GH_min
     REAL(RealType) :: GH_max
     REAL(RealType) :: total_KE_Const
  END TYPE Turb_Cnts_Type

  !
  ! Parameters that can be read in from the parameter file
  !
  ! Read in using the routines in get_parameter.f90
  !
  ! Note if these parameter structures are changed/updated, then changes
  ! will need to be made to Read_Parameter_File in get_parameters.f90
  !
  !
  ! Note also not all parameters are listed as there is some decision making
  ! regarding input file type (if ASCII need lat/lon grid size etc. plus two) 
  ! input files etc.
  !
  INTEGER(Int4), PARAMETER :: nParameters = 32
  CHARACTER(LEN=80) :: Parameter_names(nParameters) = (/&
     "days                        ",&
     "print_freq_hours            ",&
     "nlevels                     ",&
     "dt1                         ",&
     "smooth                      ",&
     "FCT                         ",&
     "bottom_depth                ",&
     "bottom_salinity             ",&
     "KMB_init                    ",&
     "KHB_init                    ",&
     "shear_enhancement_factor    ",&
     "wave_enhancement            ",&
     "surface_abs                 ",&
     "modify_fluxes               ",&
     "use_langmuir                ",&
     "exponential_damping         ",&
     "use_ascii_input             ",&
     "shear_flag                  ",&
     "wave_breaking               ",&
     "use_evaporation             ",&
     "File_Turb_Cnts              ",&
     "Output_File                 ",&
     "test_output                 ",&
     "Output_File_NetCDF_Format   ",&
     "Use_Solid_Bottom            ",&
     "Solid_Bottom_Temp           ",&
     "use_skin_temperature        ",&
     "output_ascii_profiles       ",&
     "output_file_profile         ",&
     "langmuir_top_boundary_factor",&
     "reference_depth             ",&
     "use_estimated_wavedata      "&
     /)

  !
  ! Type of parameter - used in get_parameter.f90
  !
  CHARACTER(LEN=10) :: Parameter_VarType(nParameters) = (/&
     "REAL     ",&
     "REAL     ",&
     "INT      ",&
     "REAL     ",&
     "REAL     ",&
     "REAL     ",&
     "REAL     ",&
     "REAL     ",&
     "REAL     ",&
     "REAL     ",&
     "REAL     ",&
     "REAL     ",&
     "LOGICAL  ",&
     "LOGICAL  ",&
     "LOGICAL  ",&
     "LOGICAL  ",&
     "LOGICAL  ",&
     "LOGICAL  ",&
     "LOGICAL  ",&
     "LOGICAL  ",&
     "CHARACTER",&
     "CHARACTER",&
     "LOGICAL  ",&
     "LOGICAL  ",&
     "LOGICAL  ",&
     "REAL     ",&
     "LOGICAL  ",&
     "LOGICAL  ",&
     "CHARACTER",&
     "REAL     ",&
     "REAL     ",&
     "LOGICAL  "&
     /)

  !
  ! Structure containing all the parameter values and derived values
  !
  ! These are read in from a parameter file, decoded in get_parameter.f90
  !
  TYPE Parameter_Type
     LOGICAL :: read_in_params
     LOGICAL :: date_there
     INTEGER(Int4) :: Year
     INTEGER(Int4) :: Month
     INTEGER(Int4) :: Day
     REAL(RealType) :: days
     REAL(RealType) :: print_freq_hours
     INTEGER(Int4) :: print_freq_loop
     INTEGER(Int4) :: nlevels
     INTEGER(Int4) :: ngrid
     REAL(RealType) :: dt1
     REAL(RealType) :: dt2
     REAL(RealType) :: smooth
     REAL(RealType) :: FCT
     REAL(RealType) :: bottom_depth
     REAL(RealType) :: bottom_salinity
     REAL(RealType) :: KMB_init
     REAL(RealType) :: KHB_init
     REAL(RealType) :: shear_enhancement_factor
     REAL(RealType) :: wave_enhancement
     LOGICAL :: surface_abs
     LOGICAL :: modify_fluxes
     LOGICAL :: use_langmuir_param
     LOGICAL :: use_langmuir
     LOGICAL :: exponential_damping
     LOGICAL :: use_ascii_input
     LOGICAL :: shear_flag
     LOGICAL :: wave_breaking
     LOGICAL :: use_evaporation
     CHARACTER(LEN=256) :: File_Turb_Cnts
     CHARACTER(LEN=256) :: File_Grid
     CHARACTER(LEN=256) :: File_Netcdf
     CHARACTER(LEN=256) :: File_Fluxes
     CHARACTER(LEN=256) :: Output_File
     LOGICAL :: test_output
     LOGICAL :: Output_File_NetCDF_Format
     LOGICAL :: Use_Solid_Bottom
     REAL(RealType) :: Solid_Bottom_Temp
     LOGICAL :: use_skin_temperature
     LOGICAL :: output_ascii_profiles
     CHARACTER(LEN=256) :: output_file_profile
     REAL(RealType) :: langmuir_top_boundary_factor
     INTEGER(Int4) :: grid_nx
     INTEGER(Int4) :: grid_ny
     REAL(RealType) :: grid_min_lon
     REAL(RealType) :: grid_max_lon
     REAL(RealType) :: grid_min_lat
     REAL(RealType) :: grid_max_lat
     REAL(RealType) :: lat_step
     REAL(RealType) :: lon_step
     CHARACTER(LEN=8) :: DateString
     REAL(RealType) :: reference_depth
     LOGICAL :: use_estimated_wavedata
  END TYPE Parameter_Type
  
  !
  ! Band coefficients for radiative absorption
  !
  INTEGER, PARAMETER :: max_nbands = 9
  TYPE Rad_Bands_Type
     INTEGER(Int4) :: nbands
     REAL(RealType) :: alpha(max_nbands)
     REAL(RealType) :: scl(max_nbands)     
     REAL(RealType) :: scli(max_nbands)     
     LOGICAL :: there
  END TYPE Rad_Bands_Type

  !
  ! Surface flux structure
  !
  TYPE Surface_Type
     !
     ! position data etc.
     !
     INTEGER(Int4) :: date
     REAL(RealType) :: latitude
     REAL(RealType) :: longitude
     REAL(RealType) :: sstinit
     INTEGER(Int4) :: i_out
     INTEGER(Int4) :: j_out
     !
     ! Model data time step 1
     !
     REAL(Dble) :: time1
     REAL(RealType) :: wind_stress_x1
     REAL(RealType) :: wind_stress_y1
     REAL(RealType) :: downwelling_solar1
     REAL(RealType) :: heat_flux_longwave1
     REAL(RealType) :: heat_flux_latent1
     REAL(RealType) :: heat_flux_sensible1
     REAL(RealType) :: significant_wave_height1
     REAL(RealType) :: wave_period1
     REAL(RealType) :: wave_dir1
     REAL(RealType) :: model_sst1   ! Note model SST is in Centigrade
     REAL(RealType) :: temp_atmos1
     REAL(RealType) :: specific_humidity_atmos1
     REAL(RealType) :: pressure1
     !
     ! Model data time step 2
     !
     REAL(Dble) :: time2
     REAL(RealType) :: wind_stress_x2
     REAL(RealType) :: wind_stress_y2
     REAL(RealType) :: downwelling_solar2
     REAL(RealType) :: heat_flux_longwave2
     REAL(RealType) :: heat_flux_latent2
     REAL(RealType) :: heat_flux_sensible2
     REAL(RealType) :: significant_wave_height2
     REAL(RealType) :: wave_period2
     REAL(RealType) :: wave_dir2
     REAL(RealType) :: model_sst2   ! Note model SST is in Centigrade
     REAL(RealType) :: temp_atmos2
     REAL(RealType) :: specific_humidity_atmos2
     REAL(RealType) :: pressure2
     !
     ! Time weighted to current time values
     !
     REAL(Dble) :: start_time
     REAL(Dble) :: time
     INTEGER(Int4) :: Year
     REAL(RealType) :: wind_stress_x
     REAL(RealType) :: wind_stress_y
     REAL(RealType) :: downwelling_solar
     REAL(RealType) :: heat_flux_longwave
     REAL(RealType) :: heat_flux_latent
     REAL(RealType) :: heat_flux_sensible
     REAL(RealType) :: significant_wave_height
     REAL(RealType) :: wave_period
     REAL(RealType) :: wave_dir
     REAL(RealType) :: model_sst
     REAL(RealType) :: temp_atmos
     REAL(RealType) :: specific_humidity_atmos
     REAL(RealType) :: pressure
     !
     ! Calculated parameters 
     !
     REAL(RealType) :: clear_sky_index1
     REAL(RealType) :: clear_sky_index2
     REAL(RealType) :: qsol
     REAL(RealType) :: Ustar
     REAL(RealType) :: Ustar_atmos
     REAL(RealType) :: wind_speed
     REAL(RealType) :: WU_surf
     REAL(RealType) :: WV_surf
     REAL(RealType) :: breakwave_turb_scale
     !
     ! Radiation
     !
     REAL(RealType) :: net_solar
     REAL(RealType) :: net_longwave
     REAL(RealType) :: flux_total
     !
     ! Salinity
     !
     REAL(RealType) :: salinity_flux
     REAL(RealType) :: evaporation
     REAL(RealType) :: precipitation
     !
     ! Skin temperature
     !
     REAL(RealType) :: tskin
     !
     ! Flag
     !
     LOGICAL :: surface_there
     LOGICAL :: fluxes_init
  END type Surface_Type

  !
  ! Profile of meaningful values including forward and backward components
  !
  TYPE Profile_Type
     !
     ! Number of levels and associated params
     !
     INTEGER(Int4) :: nlevels
     INTEGER(Int4) :: nlevels_minus_1
     INTEGER(Int4) :: nlevels_minus_2
     INTEGER(Int4) :: pos_reference
     REAL(RealType) :: bottom      ! Depth of bottom
     REAL(RealType) :: bottom_sq   ! Depth of bottom squared
     LOGICAL :: profile_allocated  ! Check if structure allocated
     !
     ! Depth arrays
     !
     LOGICAL :: depth_there  ! Have depth arrays been defined
     REAL(RealType), ALLOCATABLE :: z(:)
     REAL(RealType), ALLOCATABLE :: zz(:)
     REAL(RealType), ALLOCATABLE :: delta_z(:)
     REAL(RealType), ALLOCATABLE :: delta_zz(:)
     !
     ! Density
     !
     REAL(RealType), ALLOCATABLE :: rho(:)
     !
     ! Radiation at each level
     !
     ! Note: absorption just a function of depth which doesn't change 
     ! at different locations (in this version depth profile doesn't change)
     ! so it is set when the depth profile is assigned
     !
     REAL(RealType), ALLOCATABLE :: swabs(:)
     !
     ! Here is the array containing the radiation at depth which does change
     !
     REAL(RealType), ALLOCATABLE :: swrad(:)
     !
     ! DM - part of the equation of motion
     ! DM_TEMP - DM for the temperature retrieval part
     !
     REAL(RealType), ALLOCATABLE :: dm(:)     
     REAL(RealType), ALLOCATABLE :: dm_temp(:)     
     !
     ! Temperature profiles - _f for forward in time, _b for backward in time
     !
     LOGICAL :: temperature_there
     REAL(RealType), ALLOCATABLE :: Temp(:)
     REAL(RealType), ALLOCATABLE :: Temp_f(:)     
     REAL(RealType), ALLOCATABLE :: Temp_b(:)
     !
     ! Salinity
     !
     LOGICAL :: salinity_there
     REAL(RealType), ALLOCATABLE :: Salinity(:)
     REAL(RealType), ALLOCATABLE :: Salinity_f(:)
     REAL(RealType), ALLOCATABLE :: Salinity_b(:)
     !
     ! Velocities
     !
     LOGICAL :: velocity_there
     !
     ! V component
     !
     REAL(RealType), ALLOCATABLE :: Velocity_V(:)
     REAL(RealType), ALLOCATABLE :: Velocity_V_f(:)
     REAL(RealType), ALLOCATABLE :: Velocity_V_b(:)
     !
     ! U component
     !
     REAL(RealType), ALLOCATABLE :: Velocity_U(:)
     REAL(RealType), ALLOCATABLE :: Velocity_U_f(:)
     REAL(RealType), ALLOCATABLE :: Velocity_U_b(:)
     !
     ! Bottom boundary conditions
     !
     REAL(RealType) :: WU_Bottom
     REAL(RealType) :: WV_Bottom
     !
     ! Stokes drift velocities
     !
     REAL(RealType), ALLOCATABLE :: Velocity_U_Stokes(:)
     REAL(RealType), ALLOCATABLE :: Velocity_V_Stokes(:)
     !
     ! Turbulent Kinetic Energy
     !
     REAL(RealType), ALLOCATABLE :: q2(:)
     REAL(RealType), ALLOCATABLE :: q2_f(:)
     REAL(RealType), ALLOCATABLE :: q2_b(:)
     !
     ! Q2 x length
     !
     REAL(RealType), ALLOCATABLE :: q2l(:)
     REAL(RealType), ALLOCATABLE :: q2l_f(:)
     REAL(RealType), ALLOCATABLE :: q2l_b(:)
     !
     ! qli
     !
     REAL(RealType), ALLOCATABLE :: qli(:)
     !
     ! Turbulent length
     !
     REAL(RealType), ALLOCATABLE :: L(:)
     !
     ! g d(rho)/dz
     ! q2 x turnulence length
     !
     REAL(RealType), ALLOCATABLE :: g_drho_dz(:)
     !
     ! Richardson No.
     !
     REAL(RealType), ALLOCATABLE :: Richardson_no(:)
     !
     ! Production variables
     !
     REAL(RealType), ALLOCATABLE :: Shear_prod(:)
     REAL(RealType), ALLOCATABLE :: Buoyancy_prod(:)
     REAL(RealType), ALLOCATABLE :: Shear_prod_stokes(:)
     REAL(RealType), ALLOCATABLE :: Eddy_Diss_q2l(:)
     REAL(RealType), ALLOCATABLE :: prod(:)
     !
     ! Diffusivity terms
     !
     REAL(RealType), ALLOCATABLE :: KH(:)
     REAL(RealType), ALLOCATABLE :: KH_B(:)
     REAL(RealType), ALLOCATABLE :: KM(:)
     REAL(RealType), ALLOCATABLE :: KM_B(:)
     REAL(RealType), ALLOCATABLE :: KQ(:)
     REAL(RealType), ALLOCATABLE :: KN(:)
     !
     ! GH
     !
     REAL(RealType), ALLOCATABLE :: GH(:)
     !
     ! GH
     !
     REAL(RealType), ALLOCATABLE :: SH(:)
     REAL(RealType), ALLOCATABLE :: SM(:)
     !
     ! Is profile initialized
     !
     LOGICAL :: initialized
  END TYPE Profile_Type


END MODULE Types
