!=================================================================
!
! NAME: 
!
!    depth
!
! FUNCTION:
!
!    Calcualtes the depth profile and associated variables 
!
! TYPE/LANGUAGE: 
!
!    F90 module
!
! DESCRIPTION:  
!
!    Calcalated the depth profile (scaled to bottom) and mid points, layer
!    thicknesses etc.
!
! ROUTINES TO BE USED EXTERNALLY:
!
!    Get_Depth_Scale_Oper      - Gets the depth scale
!
! ROUTINES USED INTERNALLY:
!
! HISTORY:
! 
!   Original version by Jonathan P.D. Mittaz on 9/18/2013
!
!=================================================================
MODULE Depth_Routines

  USE Kinds
  USE Types
  USE ErrorHandler

  IMPLICIT NONE

  !
  ! In structure Profile_Str the following apply
  !
  ! Z is the depth
  ! ZZ is the midpoint between the levels
  ! delta_Z is the thickness between the levels
  ! delta_ZZ is the thickness between the midpoints (layers)
  ! swabs is the amount of absorption at depth given rad bands values
  !

  !
  ! Make sure other routines only see what we want them to
  !
  PRIVATE
  PUBLIC :: Get_Depth_Scale_oper

CONTAINS
  
  !=================================================================
  !
  ! NAME:
  !
  !      Get_Depth_Scale_oper
  !
  ! FUNCTION:
  !
  !      Sets the depth values which the main code works on
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Sets the depth scales that the code uses for the modelling.  This
  !      is the scaling currently defined to be the operational one
  !
  !      Note that this code is written so that it can be changed easily
  !      and the data (number of levels etc.) is propagated to the rest
  !      of the code transparently.  Other structures are also allocated in 
  !      this routine if they match the depth profile.
  !
  !      NOTE
  ! 
  !      If the input bottom and nlevels are as fixed in the original code
  !      (bottom = 50m, nlevels = 140) then code does the z levels the same
  !      way.  If not, the code scales the numbers to give a fixed top 5
  !      levels, and exponential variable to 15m and linear after that
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  !
  ! CALLING SEQUENCE:
  !
  !     CALL Get_Depth_Scale( params, rad_bands, profile )
  !
  ! INPUTS:
  !
  !     params                - inpul parameters
  !     rad_bands             - solar absorption bands information
  ! 
  ! OUTPUTS:
  !
  !     profile               - structure that contains the depth information
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Based on code supplied by Gary Wick NOAA/OAR
  !
  !      Original F90 version by Jonathan P.D. Mittaz 8/14/2013
  !      
  !
  SUBROUTINE Get_Depth_Scale_Oper( params, rad_bands, profile )

    TYPE(Parameter_Type), INTENT(IN) :: params
    TYPE(Rad_Bands_Type), INTENT(IN) :: rad_bands
    TYPE(Profile_Type), INTENT(INOUT) :: profile

    !
    ! Local variables
    !
    INTEGER(Int4) :: K
    INTEGER(Int4) :: N
    INTEGER(Int4) :: max_loop

    INTEGER(Int4) :: ipos1
    INTEGER(Int4) :: ipos2

    REAL(RealType) :: diff
    REAL(RealType) :: mindiff
    REAL(RealType) :: scale1
    REAL(RealType) :: scale2
    REAL(RealType) :: offset2
    REAL(RealType) :: offset3
    
    !
    ! Check that profile has been allocated
    !
    IF( .NOT.profile%profile_allocated )THEN
       CALL out_ERROR('Profile structure not allocated',&
            'Get_Depth_Scale_Oper','depth.f90')
    ENDIF

    !
    ! Scale from original values which assumed a bottom of 50m
    ! Keep break points in m the same - same gridding up to z(105)*50.
    ! which is 15m point
    !
    IF( params%bottom_depth .eq. 50._RealType &
         .and. params%nlevels .eq. 140 )THEN
       !
       ! These are the original values based on 140 levels and a bottom of 
       ! 50m
       !
       !
       ! Make depth values
       !
       DO K=1,5
          profile%z(K)=-0.005_RealType*REAL(K-1)
       END DO
       
       DO K=6,105
          profile%z(K)=-(exp(0.027723_RealType*REAL(K-5))-0.995_RealType)
       END DO
       
       DO K=106,params%nlevels
          profile%z(K)=-1.0*REAL(K-90)
       END DO

    ELSE
       ipos1 = 5
       scale1 = 0.005_RealType
       IF( params%bottom_depth .ge. 15._RealType )THEN
          ! 
          ! first level at 0.025m
          ! second level at 15m
          !
          ipos2 = NINT((100._RealType/140._RealType)*params%nlevels)
          IF( 1 .ge. ipos2 )THEN
             CALL out_ERROR('Not enough levels defined',&
                  'Get_Depth_Scale_Oper','depth.f90')
          ENDIF
          offset2 = 0.975_RealType
          scale2 = LOG(15.0_RealType+offset2)/(ipos2-1)
          ipos2 = ipos2 + 5
          offset3 = (params%bottom_depth - &
               0.025_RealType+(exp(scale2*(ipos2-6))-offset2))/&
               (params%nlevels-ipos2)
       ELSE
          ipos2 = params%nlevels
          offset2 = 0.975_RealType
          scale2 = LOG(params%bottom_depth+offset2)/(ipos2-1)
       ENDIF

       DO K=ipos1+1,ipos2
          profile%z(K) = exp(scale2*(K-ipos1-1)) - offset2
       END DO

       ! From this point if applicable - 1m steps
       IF( ipos2 .lt. params%nlevels )THEN
          DO K=ipos2+1,params%nlevels
             profile%z(K)=profile%z(K-1) - offset3             
          END DO
       ENDIF

    ENDIF
    
    !
    ! Find reference level
    !
    profile%pos_reference = -1
    mindiff = 1e30_RealType
    DO K=1,params%nlevels
       diff = ABS(profile%z(K)-params%reference_depth)
       IF( diff .lt. mindiff )THEN
          mindiff = diff
          profile%pos_reference = K
       ENDIF
    END DO
    IF( -1 .eq. profile%pos_reference )THEN
       CALL out_ERROR('reference depth level does not exist - cannot run model',&
            'Get_Depth_Scale_Oper','depth.f90')
    ENDIF

    !
    ! Scale to bottom value
    !
    DO K=1,params%nlevels
       profile%z(K) = profile%z(K)/params%bottom_depth
    END DO

    !
    ! Now get midpoint values and thicknesses
    !
    DO K=1,params%nlevels-1
       profile%zz(K)=(profile%z(K)+profile%z(K+1))*0.5_RealType
    END DO
    DO K=1,params%nlevels-1
       profile%delta_z(K)=profile%z(K)-profile%z(K+1)
       profile%delta_zz(K)=profile%zz(K)-profile%zz(K+1)
    END DO
    profile%zz(params%nlevels)=-1.01_RealType
    profile%delta_z(params%nlevels)=0.01_RealType
    profile%delta_zz(params%nlevels)=0.01_RealType
    profile%delta_zz(params%nlevels-1)=0.01_RealType

    ! 
    ! Now using the radbands calculate the absorption at depth
    !
    ! Do it here as we only need to do this once and it only depends on 
    ! the depth information
    !
    !
    ! If surface absorption to be allowed
    !
    IF( .not.rad_bands%there )THEN
       CALL out_ERROR('Rad bands constants have not been set',&
            'Get_Depth_Scale_Oper','depth.f90')
    ENDIF

    IF( params%surface_abs )THEN
       max_loop = rad_bands%nbands-1
    ELSE
       max_loop = rad_bands%nbands
    ENDIF
    
    !
    ! Calculate the absorption
    !
    profile%swabs = 0.0_RealType
    DO K=1,params%nlevels
       DO N=1,max_loop
          profile%swabs(K) = profile%swabs(K) + rad_bands%alpha(N) * &
               EXP(profile%z(K)*profile%bottom*rad_bands%scli(N))
       END DO
    END DO

    !
    ! Depth arrays have been defined
    !
    profile%depth_there = .TRUE.

  END SUBROUTINE Get_Depth_Scale_oper

END MODULE Depth_Routines
