!=================================================================
!
! NAME: 
!
!    Get_Data
!
! FUNCTION:
!
!    Reads in model and assiated data
!
! TYPE/LANGUAGE: 
!
!    F90 module2
!
! DESCRIPTION:  
!
!    Reads in initial SST and model data and time interpolates model data 
!    to time step
!
! ROUTINES TO BE USED EXTERNALLY
!
!    Access_Data_Files    - Open model data files and read in parameters
!                           from parameter file as well as the turbulent
!                           constants
!    Close_Data_Files     - Close all open model files
!    Read_in_grid         - Reads in grid (sst init data)
!    Get_Fluxes           - Reads in model flux data including wind model
!
! ROUTINES USED INTERNALLY:
!
!    Read_Turbulence_File - Reads in turbulence data
!    Modify_Fluxes        - Modifies the flux data on the basis of any 
!                           diurnal warming
!    humidity             - Get the humidity at the surface
!    Get_Stokes           - Get stokes drift information based on wind model
!
! HISTORY:
! 
!   Original version by Jonathan P.D. Mittaz on 9/10/2013
!
!=================================================================
MODULE Get_Data

  USE Kinds
  USE Types
  USE Constants
  USE Solar_Routines
  USE ErrorHandler
  USE get_parameters
  USE profile
  USE Output_Values

  IMPLICIT NONE

  !
  ! Global variables for this module
  !
  INTEGER(Int4) :: Unit_Grid
  INTEGER(Int4) :: Unit_Fluxes

  !
  ! Storage for NetCDF read in data
  !
  ! Used to emulate Gary Wicks ASCII file input
  !
  ! Position of cell to be worked on
  !
  INTEGER(Int4) :: store_xpos = 1
  INTEGER(Int4) :: store_ypos = 1
  INTEGER(Int4) :: store_tpos = 1
  !
  ! Structure containing netcdf read in data
  !
  TYPE Store_NETCDF_Type
     INTEGER(Int4) :: ntime
     INTEGER(Int4) :: nlat
     INTEGER(Int4) :: nlon
     REAL(RealType), ALLOCATABLE :: latitude(:)
     REAL(RealType), ALLOCATABLE :: longitude(:)
     REAL(RealType), ALLOCATABLE :: land(:,:)
     REAL(RealType), ALLOCATABLE :: ice(:,:,:)
     REAL(Dble), ALLOCATABLE :: frac_day(:)
     REAL(RealType), ALLOCATABLE :: wind_stress_x(:,:,:)
     REAL(RealType), ALLOCATABLE :: wind_stress_y(:,:,:)
     REAL(RealType), ALLOCATABLE :: downwelling_solar(:,:,:)
     REAL(RealType), ALLOCATABLE :: heat_flux_longwave(:,:,:)
     REAL(RealType), ALLOCATABLE :: heat_flux_latent(:,:,:)
     REAL(RealType), ALLOCATABLE :: heat_flux_sensible(:,:,:)
     REAL(RealType), ALLOCATABLE :: significant_wave_height(:,:,:)
     REAL(RealType), ALLOCATABLE :: wave_period(:,:,:)
     REAL(RealType), ALLOCATABLE :: wave_dir(:,:,:)
     REAL(RealType), ALLOCATABLE :: model_sst(:,:,:)
     REAL(RealType), ALLOCATABLE :: temp_atmos(:,:,:)
     REAL(RealType), ALLOCATABLE :: specific_humidity_atmos(:,:,:)
     REAL(RealType), ALLOCATABLE :: pressure(:,:,:)
  END type Store_NETCDF_Type

  !
  ! This is where the data is stored - global (internal) variable
  !
  TYPE(Store_NETCDF_Type) :: netcdf_store

  PRIVATE
  PUBLIC :: Access_Data_Files
  PUBLIC :: Close_Data_Files
  PUBLIC :: Read_in_grid
  PUBLIC :: Get_Fluxes

CONTAINS
  
  !=================================================================
  !
  ! NAME:
  !
  !      Access_Data_Files
  !
  ! FUNCTION:
  !
  !      Gets input files from the command line and reads them into the
  !      relevant structures.  
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Gets the required input files from the command line and reads the
  !      the paramater file as well as the turbulence parameters.  Also 
  !      opens the data streams for the input fluxes.
  ! 
  !      This version reads the original versions of the file generated by
  !      Gary Wicks IDL code, not the new format files.  
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  !      IARGC
  !      GETARG
  !      Read_Parameter_File
  !      Read_Turbulence_File
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Access_Data_Files( params, turb_cnts )
  !
  ! INPUTS:
  ! 
  ! OUTPUTS:
  !
  !      params    - Parameters needed to run the code
  !      turb_cnts - Turbulence constants
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Based on code supplied by Gary Wick NOAA/OAR
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !      
  !
  SUBROUTINE Access_Data_Files( params, turb_cnts )

    TYPE(parameter_type), INTENT(OUT) :: params
    TYPE(Turb_Cnts_Type), INTENT(OUT) :: turb_cnts

    ! Local variables
    INTEGER(Int4) :: nargs
    INTEGER(Int4) :: IOS
    CHARACTER(LEN=256) :: File_parameter
    CHARACTER(LEN=256) :: Model_Dir
    CHARACTER(LEN=256) :: DateString

    !
    ! Get filenames from command file
    !
    ! Check number of arguments
    !
    nargs = IARGC()
    IF( 3 .ne. nargs )THEN
       WRITE(*,'('' USAGE:'')')
       WRITE(*,'(''        kantha_clayson_dwmodel.exe param_file model_dir date_string'')')
       WRITE(*,'(''            param_file      - Input parameter file'')')
       WRITE(*,'(''            model_dir       - Directory for model input/ouput'')')
       WRITE(*,'(''            date_string     - Date string (YYYYMMDD)'')')
       WRITE(*,'('' '')')
       WRITE(*,'('' OR '')')
       WRITE(*,'('' '')')
       WRITE(*,'(''        kantha_clayson_dwmodel.exe param_file model_dir model_file'')')
       WRITE(*,'(''            param_file      - Input parameter file'')')
       WRITE(*,'(''            model_dir       - Directory for model input/ouput'')')
       WRITE(*,'(''            model_file      - Filename of model NetCDF file'')')
       STOP 1
    ENDIF

    !
    ! Get parameter filename
    !
    CALL GETARG(1,File_parameter)

    !
    ! Get directory where model data will be stored
    !
    CALL GETARG(2,Model_Dir)

    ! 
    ! Get Date string for file input/output
    !
    CALL GETARG(3,DateString)

    !
    ! Read in parameter file
    !
    CALL Read_Parameter_File(File_parameter,params,Model_Dir,DateString)

    !
    ! Read in turbulence parameters file
    !
    CALL Read_Turbulence_File(params%File_turb_cnts,params,turb_cnts)

    !
    ! Open Model data
    !
    IF( params%use_ascii_input )THEN
       !
       ! Open grid file
       !
       Unit_Grid = Get_File_Unit()
       OPEN(Unit=Unit_Grid,FILE=params%File_grid,STATUS='OLD',IOSTAT=IOS)
       IF( 0 .ne. IOS )THEN
          CALL out_ERROR('Cannot open GRID file','Access_Data_Files',&
               'get_data.f90')
       ENDIF
       
       !
       ! Open Fluxes file
       !
       Unit_fluxes = Get_File_Unit()
       OPEN(Unit=Unit_fluxes,FILE=params%File_fluxes,STATUS='OLD',IOSTAT=IOS)
       IF( 0 .ne. IOS )THEN
          CALL out_ERROR('Cannot open FLUXES file','Access_Data_Files',&
               'get_data.f90')
       ENDIF

       !
       ! Open output file
       !
       CALL Set_Output_File(params)
    
    ELSE

       !
       ! Read in NETCDF file in one go
       !
       CALL Read_netCDF_Model(params)

       !
       ! Open output file
       !
       CALL Set_Output_File(params)
    
    ENDIF

  END SUBROUTINE Access_Data_Files

  !=================================================================
  !
  ! NAME:
  !
  !      Read_NetCDF_Model
  !
  ! FUNCTION:
  !
  !      Reads in the model data plus wave model from NetCDF file
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Reads in the GFS fluxes and the wave model data from a NetCDF
  !      file.  Loads data into internal storage to be 'read' later
  !
  !      Note: Changes the input time to match old ASCII format used 
  !      for in-situ testing and deals with year boundaries.
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  !      out_ERROR
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Read_NetCDF_Model( params )
  !
  ! INPUTS:
  !
  !      params    - parameter file
  !
  ! OUTPUTS:
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Based on code supplied by Gary Wick NOAA/OAR
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !      
  !
  SUBROUTINE Read_netCDF_Model( params )    

    USE NetCDF

    Type(Parameter_TYPE), INTENT(INOUT) :: params

    ! Local variables
    INTEGER(Int4) :: I
    INTEGER(Int4) :: status
    INTEGER(Int4) :: ncid
    INTEGER(Int4) :: varid
    INTEGER(Int4) :: time_dimid
    INTEGER(Int4) :: latitude_dimid
    INTEGER(Int4) :: longitude_dimid
    INTEGER(Int4) :: nlon
    INTEGER(Int4) :: nlat
    INTEGER(Int4) :: ntime

    INTEGER(Int4) :: min_year
    INTEGER(Int4) :: max_year
    INTEGER(Int4) :: year_tot
    INTEGER(Int4), ALLOCATABLE :: day_array(:)
    INTEGER(Int4), ALLOCATABLE :: year(:)

    CHARACTER(LEN=256) :: dimname

    !
    ! Check that parameter file has been read in - don't want a read to overwrite some of the 
    ! parameters that are redefined here
    !
    IF( .not.params%read_in_params )THEN
       CALL out_ERROR('Parameter file not read in','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    WRITE(*,'('' Reading in NetCDF file : '',a)')TRIM(params%file_netcdf)

    !
    ! Open NetCDF file
    !
    status = nf90_open(params%file_netcdf,NF90_NOWRITE,ncid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot open model NetCDF file','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! Get dimensions
    !
    status = nf90_inq_dimid(ncid,'time',time_dimid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get time dimension in model netcdf file',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_inq_dimid(ncid,'latitude',latitude_dimid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get latitude dimension in model netcdf file',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_inq_dimid(ncid,'longitude',longitude_dimid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get longitude dimension in model netcdf file',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF

    !
    ! Get length of dimensions
    !
    status = nf90_inquire_dimension(ncid,time_dimid,dimname,netcdf_store%ntime)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR(&
            'Cannot get length of time dimension in model netcdf file',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_inquire_dimension(ncid,longitude_dimid,dimname,&
         netcdf_store%nlon)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR(&
            'Cannot get length of longitude dimension in model netcdf file',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_inquire_dimension(ncid,latitude_dimid,dimname,&
         netcdf_store%nlat)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR(&
            'Cannot get length of latitude dimension in model netcdf file',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF

    !
    ! Allocate all storage arrays
    !
    nlon = netcdf_store%nlon
    nlat = netcdf_store%nlat
    ntime = netcdf_store%ntime
    !
    ! Set params value (used to set output)
    !
    params%grid_nx = nlon
    params%grid_ny = nlat
    params%ngrid = nlon*nlat
    !
    ! Allocate storage arrays
    !
    ALLOCATE(netcdf_store%frac_day(ntime),&
         netcdf_store%latitude(nlat),&
         netcdf_store%longitude(nlon),&
         netcdf_store%land(nlon,nlat),&
         netcdf_store%ice(nlon,nlat,ntime),&
         netcdf_store%wind_stress_x(nlon,nlat,ntime),&
         netcdf_store%wind_stress_y(nlon,nlat,ntime),&
         netcdf_store%model_sst(nlon,nlat,ntime),&
         netcdf_store%heat_flux_longwave(nlon,nlat,ntime),&
         netcdf_store%downwelling_solar(nlon,nlat,ntime),&
         netcdf_store%heat_flux_latent(nlon,nlat,ntime),&
         netcdf_store%heat_flux_sensible(nlon,nlat,ntime),&
         netcdf_store%significant_wave_height(nlon,nlat,ntime),&
         netcdf_store%wave_period(nlon,nlat,ntime),&
         netcdf_store%wave_dir(nlon,nlat,ntime),&
         netcdf_store%temp_atmos(nlon,nlat,ntime),&
         netcdf_store%specific_humidity_atmos(nlon,nlat,ntime),&
         netcdf_store%pressure(nlon,nlat,ntime),&
         STAT=status)
    IF( 0 .ne. status )THEN
       CALL out_ERROR('Cannot allocate arrays for NetCDF model file',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF

    !
    ! Read in data
    !
    ! Time variable - this is stored as frac_day (YYYYDDD.Fraction)
    !
    status = nf90_inq_varid(ncid,'frac_day',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for time','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%frac_day)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get time values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! Get maximum number of days from frac_day
    !
    ALLOCATE(day_array(ntime),year(ntime),STAT=status)
    IF( 0 .ne. status )THEN
       CALL out_ERROR('Cannot allocate day_array/year',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    !
    ! Get year range
    !
    min_year = 10000
    max_year = -10000
    DO I=1,ntime
       year(I) = INT(netcdf_store%frac_day(I)/1000.0_Dble)
       day_array(I) = netcdf_store%frac_day(I) - year(I)*1000.0_Dble
       min_year = MIN(min_year,year(I))
       max_year = MAX(max_year,year(I))
    END DO

    !
    ! If cross year boundary then deal with it
    !
    IF( min_year .ne. max_year )THEN
       IF( isLeap(min_year) )THEN
          year_tot = 366
       ELSE
          year_tot = 365
       ENDIF
       DO I=1,ntime
          IF( year(I) .ne. min_year )THEN
             day_array(I) = day_array(I) + year_tot
          ENDIf
       END DO
    ENDIF

    !
    ! Assume that the input is ordered in increasing time (which it should be)
    !
    IF( params%days .gt. day_array(ntime)-day_array(1) )THEN
       !
       ! make sure that we only do analysis where we have input data
       !
       params%days = day_array(ntime)-day_array(1)       
    ELSE IF( params%days .lt. day_array(ntime)-day_array(1) )THEN
       !
       ! make sure that we only do analysis we need an output from
       !
       timeLoop: DO I=1,ntime
          IF( params%days .lt. day_array(I)-day_array(1) )THEN
             ntime = I-1
             netcdf_store%ntime = ntime
          ENDIF
       END DO timeLoop
    ENDIF
    !
    ! Deallocate temporary arrays
    !
    DEALLOCATE(day_array,year,STAT=status)
    IF( 0 .ne. status )THEN
       CALL out_ERROR('Cannot deallocate day_array/year',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    !
    ! Check params%days is >= day range
    !

    !
    ! Latitude variable
    !
    status = nf90_inq_varid(ncid,'latitude',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for latitude','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%latitude)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get latitude values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! Now get latitude/longitude and use this grid as the output grid
    !
    
    ! 
    ! Get minLat
    !
    params%grid_min_lat = 1e30
    DO I=1,nlat
       params%grid_min_lat = MIN(params%grid_min_lat,netcdf_store%latitude(I))
    END DO

    ! 
    ! Get maxLat
    !
    params%grid_max_lat = -1e30
    DO I=1,nlat
       params%grid_max_lat = MAX(params%grid_max_lat,netcdf_store%latitude(I))
    END DO    
    
    params%lat_step = (params%grid_max_lat-params%grid_min_lat)/(nlat-1)
    params%grid_ny = nlat

    !
    ! Longitude variable
    !
    status = nf90_inq_varid(ncid,'longitude',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for longitude','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%longitude)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get longitude values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! Make sure longitude is 0,360
    !
    DO I=1,nlon
       IF( 0 .gt. netcdf_store%longitude(I) )THEN
          netcdf_store%longitude(I) = 360. + netcdf_store%longitude(I)
       ENDIF
    END DO

    ! 
    ! Get minLon
    !
    params%grid_min_lon = 1e30
    DO I=1,nlon
       params%grid_min_lon = MIN(params%grid_min_lon,netcdf_store%longitude(I))
    END DO

    ! 
    ! Get maxLon
    !
    params%grid_max_lon = -1e30
    DO I=1,nlon
       params%grid_max_lon = MAX(params%grid_max_lon,netcdf_store%longitude(I))
    END DO

    params%lon_step = (params%grid_max_lon-params%grid_min_lon)/(nlon-1)
    params%grid_nx = nlon

    !
    ! frac_day variable
    !
    status = nf90_inq_varid(ncid,'frac_day',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for frac_day','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%frac_day)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get frac_day values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! Land variable
    !
    status = nf90_inq_varid(ncid,'land',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for land','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%land)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get land values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! wind_stress_x variable
    !
    status = nf90_inq_varid(ncid,'wind_stress_x',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for wind_stress_x',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%wind_stress_x)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get wind_stress_x values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! wind_stress_y variable
    !
    status = nf90_inq_varid(ncid,'wind_stress_y',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for wind_stress_y',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%wind_stress_y)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get wind_stress_y values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! sst variable
    !
    status = nf90_inq_varid(ncid,'sst',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for sst',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%model_sst)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot sst values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! ice variable
    !
    status = nf90_inq_varid(ncid,'ice',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for ice',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%ice)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get ice values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! net_longwave variable
    !
    status = nf90_inq_varid(ncid,'net_longwave',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for net_longwave',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%heat_flux_longwave)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get net_longwave values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! net_shortwave variable
    !
    status = nf90_inq_varid(ncid,'net_shortwave',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for net_shortwave',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%downwelling_solar)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get net_shortwave values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! net_longwave variable
    !
    status = nf90_inq_varid(ncid,'latent_heat',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for latent_heat',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%heat_flux_latent)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get latent_heat','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! sensible_heat variable
    !
    status = nf90_inq_varid(ncid,'sensible_heat',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for sensible_heat',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%heat_flux_sensible)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get sensible_heat','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! air_temp_2m variable
    !
    status = nf90_inq_varid(ncid,'air_temp_2m',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for air_temp_2m',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%temp_atmos)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get air_temp_2m values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! humidity_2m variable
    !
    status = nf90_inq_varid(ncid,'humidity_2m',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for humidity_2m',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%specific_humidity_atmos)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot humidityp_2m values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF
    
    !
    ! wave_height variable
    !
    status = nf90_inq_varid(ncid,'wave_height',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for wave_height',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%significant_wave_height)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get wave_height values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! wave_period variable
    !
    status = nf90_inq_varid(ncid,'wave_period',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for wave_period',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%wave_period)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get wave_period values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! wave_direction variable
    !
    status = nf90_inq_varid(ncid,'wave_direction',varid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get varid for wave_direction',&
            'Read_NetCDF_Model','get_data.f90')
    ENDIF
    status = nf90_get_var(ncid,varid,netcdf_store%wave_dir)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot get wave_direction values','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    !
    ! surface pressure variable
    !
    status = nf90_inq_varid(ncid,'pressure',varid)
    IF( NF90_NOERR .ne. status )THEN
       !
       ! If not there, fixed
       !
       netcdf_store%pressure = 1008.0_RealType
    ELSE
       status = nf90_get_var(ncid,varid,netcdf_store%pressure)
       IF( NF90_NOERR .ne. status )THEN
          CALL out_ERROR('Cannot get pressure values','Read_NetCDF_Model',&
               'get_data.f90')
       ENDIF
    ENDIF

    !
    ! Close NetCDF file
    !
    status = nf90_close(ncid)
    IF( NF90_NOERR .ne. status )THEN
       CALL out_ERROR('Cannot close model NetCDF file','Read_NetCDF_Model',&
            'get_data.f90')
    ENDIF

    WRITE(*,'('' Read in NetCDF file '')')

  END SUBROUTINE Read_netCDF_Model

  !=================================================================
  !
  ! NAME:
  !
  !      Read_Turblence_File
  !
  ! FUNCTION:
  !
  !      Reads in the turbulence constants from a file
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Reads in the turbulence constants from a file
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  !      out_ERROR
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Read_Turbulence_File( filename, turb_cnts )
  !
  ! INPUTS:
  !
  !      filename  - input filename
  !
  ! OUTPUTS:
  !
  !      turb_cnts - Turbulence constants
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Based on code supplied by Gary Wick NOAA/OAR
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !      
  !
  SUBROUTINE Read_Turbulence_File( filename, params, turb_cnts )

    CHARACTER(LEN=*), INTENT(IN) :: filename
    TYPE(Parameter_Type), INTENT(IN) :: params
    TYPE(turb_cnts_type), INTENT(OUT) :: turb_cnts

    ! Local variables
    INTEGER :: IOS
    INTEGER :: Unit

    Unit = Get_File_Unit()
    OPEN(UNIT=Unit,FILE=filename,STATUS='OLD',IOSTAT=IOS)
    IF( 0 .ne. IOS )THEN
       CALL out_ERROR('Cannot open Turbulence File','Read_Turbulence_File',&
            'get_data.f90')
    ENDIF

    READ(Unit,*,IOSTAT=IOS)turb_cnts%A1,turb_cnts%B1,turb_cnts%A2,&
         turb_cnts%B2,turb_cnts%C1,turb_cnts%C2,turb_cnts%C3
    IF( 0 .ne. IOS )THEN
       CALL out_ERROR('Cannot read Turbulence File','Read_Turbulence_File',&
            'get_data.f90')
    ENDIF

    READ(Unit,*,IOSTAT=IOS)turb_cnts%E1,turb_cnts%E2,turb_cnts%E3,&
         turb_cnts%E4,turb_cnts%E5,turb_cnts%E6
    IF( 0 .ne. IOS )THEN
       CALL out_ERROR('Cannot read Turbulence File (line 2)',&
            'Read_Turbulence_File','get_data.f90')
    ENDIF

    READ(Unit,*,IOSTAT=IOS)turb_cnts%slsq
    IF( 0 .ne. IOS )THEN
       CALL out_ERROR('Cannot read Turbulence File (Line 3)',&
            'Read_Turbulence_File','get_data.f90')
    ENDIF
    
    CLOSE(Unit)

    !
    ! Calculate extra constants
    !
    turb_cnts%CF1 = turb_cnts%A2 * &
         (1.0_RealType-6.0_RealType*turb_cnts%A1/turb_cnts%B1)
    turb_cnts%CF2 = 3.0_RealType*turb_cnts%A2*&
         (6.0_RealType*turb_cnts%A1+turb_cnts%B2*(1.0_RealType-turb_cnts%C3))
    turb_cnts%CF3 = turb_cnts%A1*&
         (1.0_RealType-3.0_RealType*turb_cnts%C1 - &
         6.0_RealType*turb_cnts%A1/turb_cnts%B1)
    turb_cnts%CF4=9.0_RealType*turb_cnts%A1 * &
         (2.0_RealType*turb_cnts%A1+turb_cnts%A2*&
         (1.0_RealType-turb_cnts%C2))
    turb_cnts%CF5=9.0_RealType*turb_cnts%A1*turb_cnts%A2

    turb_cnts%GH_min = -0.25_RealType
    turb_cnts%GH_max = 0.98_RealType/(turb_cnts%A2*(turb_cnts%B1+&
         12.0_RealType*turb_cnts%A1 + &
         3.0_RealType*turb_cnts%B2*(1.0_RealType-turb_cnts%C3)))

    turb_cnts%total_KE_Const = 6.5073672_RealType * &
         params%shear_enhancement_factor
    
  END SUBROUTINE Read_Turbulence_File

  !=================================================================
  !
  ! NAME:
  !
  !      Close_Data_Files
  !
  ! FUNCTION:
  !
  !      Closes the model flux data files
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Closes the model flux data files.
  !
  !      Note this is for the old style (Gary Wick IDL generated) data
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Close_Data_Files(params)
  !
  ! INPUTS:
  !
  !      params   - parameter structure
  !
  ! OUTPUTS:
  !
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Based on code supplied by Gary Wick NOAA/OAR
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !      
  !
  SUBROUTINE Close_Data_Files(params)

    TYPE(Parameter_Type), INTENT(IN) :: params

    IF( params%use_ascii_input )THEN
       CLOSE(Unit_Grid)
       CLOSE(Unit_Fluxes)
    ENDIF

  END SUBROUTINE Close_Data_Files

  !=================================================================
  !
  ! NAME:
  !
  !      Read_in_Grid
  !
  ! FUNCTION:
  !
  !      Reads in the simple (lat/lon/SST) information for the flux grid
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Reads on the simple information for the flux information
  !
  !      For OLD style input always returns .TRUE.
  !      For NEW (NetCDF) returns .FALSE. when run out of grid
  !      points to use
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      STAT = Read_in_Grid(params, surface, istep)
  !
  ! INPUTS:
  !
  !      params    - input parameters
  !      istep     - time step position
  !
  ! OUTPUTS:
  !
  !      surface   - surface structure
  !
  ! RETURNS:
  !
  !      STAT      - .TRUE. if success, .FALSE. if fail
  !
  ! HISTORY
  !
  !      Based on code supplied by Gary Wick NOAA/OAR
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !      
  !
  LOGICAL FUNCTION Read_in_Grid(params,surface,istep)RESULT(Status)

    TYPE(Parameter_Type), INTENT(INOUT) :: params
    TYPE(Surface_Type), INTENT(INOUT) :: surface
    INTEGER, INTENT(IN) :: istep

    ! Local variables
    INTEGER(Int4) :: IOS
    INTEGER(Int4) :: itemp

    LOGICAL :: good_data
    LOGICAL :: test_good_data

    IF( params%use_ascii_input )THEN

       READ(Unit_grid,*,IOSTAT=IOS)surface%date,surface%latitude,&
            surface%longitude,surface%sstinit,surface%i_out,surface%j_out
       IF( 0 .ne. IOS )THEN
          CALL out_ERROR('Error reading/End of file for GRID','Read_in_Grid',&
               'get_data.f90')
       ENDIF

       !
       ! Make i_out and j_out start from an index of 1
       !
       surface%i_out = surface%i_out + 1
       surface%j_out = surface%j_out + 1

       !
       ! If date not obtained, get it 
       !
       
       surface%Year = surface%date/10000
       IF( .not.params%date_there )THEN
          ! 
          ! Get year, month, day from date
          !
          params%Year = surface%Year
          itemp = surface%date-params%Year*10000
          params%Month = itemp/100
          params%Day = itemp - params%Month*100
          
          !
          ! Get DateString
          !
          WRITE(params%DateString,'(i4.4,i2.2,i2.2)')params%Year,&
               params%Month,params%Day
          params%date_there = .TRUE.
       ENDIF

    ELSE

       !
       ! Use already read in arrays to make new values
       !
       ! Find first good data
       ! Note, the time step is not incremented here but in Get_Fluxes
       !
       IF( 1 .eq. istep )THEN
          store_tpos = 1
          store_xpos = 1
          store_ypos = 1
          !
          ! Move to first good data 
          !
          good_data = .FALSE.
          DO WHILE(.not.good_data)
             test_good_data = .FALSE.
             IF( netcdf_store%model_sst(store_xpos,store_ypos,store_tpos) .gt. &
                  -4._RealType )THEN
                IF( netcdf_store%model_sst(store_xpos,store_ypos,store_tpos) &
                     .ne. &
                  netcdf_store%model_sst(store_xpos,store_ypos,store_tpos) )THEN
                   test_good_data = .FALSE.
                ELSE
                   test_good_data = .TRUE.
                ENDIF
             ENDIF
             IF( test_good_data )THEN
                good_data = .TRUE.
             ELSE
                store_xpos = store_xpos + 1
                IF( store_xpos .gt. netcdf_store%nlon )THEN
                   store_xpos = 1
                   store_ypos = store_ypos + 1
                   IF( store_ypos .gt. netcdf_store%nlat )THEN
                      !
                      ! Ran out of data
                      !
                      Status = .FALSE.
                      RETURN
                   ENDIF
                ENDIF
             ENDIF
          END DO
       ELSE
          !
          ! Reset for next position
          !
          ! Set time index to 1
          !
          store_tpos = 1
          !
          ! Now find next good x,y position
          !
          store_xpos = store_xpos + 1
          IF( store_xpos .gt. netcdf_store%nlon )THEN
             store_xpos = 1
             store_ypos = store_ypos + 1
             IF( store_ypos .gt. netcdf_store%nlat )THEN
                !
                ! Looped round all available data
                !
                Status = .FALSE.
                RETURN
             ENDIF
          ENDIF
       ENDIF
       good_data = .FALSE.
       DO WHILE(.not.good_data)
          IF( netcdf_store%model_sst(store_xpos,store_ypos,store_tpos) .gt. &
               -4._RealType )THEN
             good_data = .TRUE.
          ELSE
             store_xpos = store_xpos + 1
             IF( store_xpos .gt. netcdf_store%nlon )THEN
                store_xpos = 1
                store_ypos = store_ypos + 1
                IF( store_ypos .gt. netcdf_store%nlat )THEN
                   !
                   ! Ran out of data
                   !
                   Status = .FALSE.
                   RETURN
                ENDIF
             ENDIF
          ENDIF
       END DO
       surface%latitude = netcdf_store%latitude(store_ypos)
       surface%longitude = netcdf_store%longitude(store_xpos)

       surface%sstinit = netcdf_store%model_sst(store_xpos,store_ypos,store_tpos)

    ENDIF

    !
    ! Make sure that even if read in ASCII version we are indexing to the correct
    ! output location - ASCII j,i index differently to NETCDF vesion...
    !
    surface%i_out = (surface%longitude - params%grid_min_lon)/params%lon_step + 1
    surface%j_out = (surface%latitude - params%grid_min_lat)/params%lat_step + 1

    !
    ! Everything OK
    !
    Status = .TRUE.
    RETURN

  END FUNCTION READ_IN_GRID

  !=================================================================
  !
  ! NAME:
  !
  !      Get_Fluxes
  !
  ! FUNCTION:
  !
  !      Reads in model data and generates all fluxes
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Reads in model data and calculates solar fluxes etc.
  !
  !      Note reads in OLD (Gary Wick) files as well as NEW (NetCDF) versions
  !      Because the solar prop routines use the old time (fractional days in
  !      year) NetCDF times are converted to that - trying to take into
  !      account year boundaries
  !
  !      If significant wave height is < 0 then assumes there is no valid
  !      awve information for this location and ignores stokes drift
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Get_Fluxes(istep,params,profile,surface)
  !
  ! INPUTS:
  !
  !      istep   - where in time iteration - if 1 then start from beginning
  !
  ! OUTPUTS:
  !
  ! INPUTS/OUTPUTS:
  !
  !      params  - input parameters - use_langmuir can be modified
  !      profile - input depth profile
  !      surface  - structure contaings surface/initialization data
  ! 
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Based on code supplied by Gary Wick NOAA/OAR
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !      
  !
  SUBROUTINE Get_Fluxes( istep, params, profile, surface )

    INTEGER, INTENT(IN) :: istep
    TYPE(parameter_type), INTENT(INOUT) :: params
    TYPE(profile_type), INTENT(INOUT) :: profile
    TYPE(surface_type), INTENT(INOUT) :: surface

    ! Local variables
    INTEGER(Int4) :: IOS
    INTEGER(Int4) :: new_year
    REAL(RealType) :: fraction
    REAL(RealType) :: sat_za
    REAL(RealType) :: tot_solar_clear_sky
    REAL(RealType) :: TauTot
    REAL(RealType) :: cli
    REAL(RealType) :: cli_dummy
    REAL(RealType) :: total_heat
    REAL(RealType) :: total_solar
    REAL(Dble) :: newtime

    !
    ! If first time - read in data
    !
    IF( 1 .eq. istep )THEN

       IF( params%use_ascii_input )THEN
          READ(Unit_Fluxes,*,IOSTAT=IOS)surface%time1,&
               surface%wind_stress_x1,surface%wind_stress_y1,&
               surface%downwelling_solar1,surface%heat_flux_longwave1,&
               surface%heat_flux_latent1,surface%heat_flux_sensible1,&
               surface%significant_wave_height1,surface%wave_period1,&
               surface%wave_dir1,surface%model_sst1,&
               surface%temp_atmos1,surface%specific_humidity_atmos1
          IF( 0 .ne. IOS )THEN
             CALL out_ERROR('End of file Fluxes file','Get_Fluxes',&
                  'get_data.f90')
          ENDIF

          READ(Unit_Fluxes,*,IOSTAT=IOS)surface%time2,&
               surface%wind_stress_x2,surface%wind_stress_y2,&
               surface%downwelling_solar2,surface%heat_flux_longwave2,&
               surface%heat_flux_latent2,surface%heat_flux_sensible2,&
               surface%significant_wave_height2,surface%wave_period2,&
               surface%wave_dir2,surface%model_sst2,&
               surface%temp_atmos2,surface%specific_humidity_atmos2
          IF( 0 .ne. IOS )THEN
             CALL out_ERROR('End of file Fluxes file','Get_Fluxes',&
                  'get_data.f90')
          ENDIF

          !
          ! For the minute have surface pressure as 1008.
          !
          surface%pressure1 = 1008.0_RealType
          surface%pressure2 = 1008.0_RealType
       
       ELSE

          !
          ! Use already read in arrays to make new values
          !
          surface%year = INT(netcdf_store%frac_day(store_tpos)/1000.0_Dble)
          surface%time1 = netcdf_store%frac_day(store_tpos) - &
               surface%year*1000.0_Dble
          surface%wind_stress_x1 = netcdf_store%wind_stress_x(store_xpos,&
               store_ypos,store_tpos)
          surface%wind_stress_y1 = netcdf_store%wind_stress_y(store_xpos,&
               store_ypos,store_tpos)
          !
          ! Note downwelling solar has to be negative but is +ve in NetCDF
          ! file
          !
          surface%downwelling_solar1 = &
               -netcdf_store%downwelling_solar(store_xpos,store_ypos,&
               store_tpos)
          surface%heat_flux_longwave1 = &
               netcdf_store%heat_flux_longwave(store_xpos,store_ypos,&
               store_tpos)
          surface%heat_flux_latent1 = &
               netcdf_store%heat_flux_latent(store_xpos,store_ypos,&
               store_tpos)
          surface%heat_flux_sensible1 = &
               netcdf_store%heat_flux_sensible(store_xpos,store_ypos,&
               store_tpos)
          surface%significant_wave_height1 = &
               netcdf_store%significant_wave_height(store_xpos,store_ypos,&
               store_tpos)
          surface%wave_period1 = netcdf_store%wave_period(store_xpos,&
               store_ypos,store_tpos)
          surface%wave_dir1 = netcdf_store%wave_dir(store_xpos,&
               store_ypos,store_tpos)
          surface%model_sst1 = netcdf_store%model_sst(store_xpos,&
               store_ypos,store_tpos)
          surface%temp_atmos1 = netcdf_store%temp_atmos(store_xpos,&
               store_ypos,store_tpos)
          surface%specific_humidity_atmos1 = &
               netcdf_store%specific_humidity_atmos(store_xpos,&
               store_ypos,store_tpos)
          surface%pressure1 = &
               netcdf_store%pressure(store_xpos,store_ypos,store_tpos)

          store_tpos = store_tpos + 1
          IF( store_tpos .gt. netcdf_store%ntime )THEN
             CALL out_ERROR('Not enough time steps stored in model data',&
                  'Get_Fluxes','get_data.f90')
          ENDIF
       
          new_year = INT(netcdf_store%frac_day(store_tpos)/1000.0_Dble)
          IF( new_year .eq. surface%year + 1)THEN
             ! Crossed a year boundary
             IF( isLeap(surface%year) )THEN
                surface%time2 = 366.0_Dble + &
                     netcdf_store%frac_day(store_tpos) - &
                     surface%year*1000.0_Dble
             ELSE
                surface%time2 = 365.0_Dble + &
                     netcdf_store%frac_day(store_tpos) - &
                     surface%year*1000.0_Dble
             ENDIF
          ELSE
             surface%time2 = netcdf_store%frac_day(store_tpos) - &
                  new_year*1000.0_Dble
          ENDIF

          surface%wind_stress_x2 = netcdf_store%wind_stress_x(store_xpos,&
               store_ypos,store_tpos)
          surface%wind_stress_y2 = netcdf_store%wind_stress_y(store_xpos,&
               store_ypos,store_tpos)
          !
          ! Note downwelling solar has to be negative but is +ve in NetCDF
          ! file
          !
          surface%downwelling_solar2 = &
               -netcdf_store%downwelling_solar(store_xpos,store_ypos,&
               store_tpos)
          surface%heat_flux_longwave2 = &
               netcdf_store%heat_flux_longwave(store_xpos,store_ypos,&
               store_tpos)
          surface%heat_flux_latent2 = &
               netcdf_store%heat_flux_latent(store_xpos,store_ypos,&
               store_tpos)
          surface%heat_flux_sensible2 = &
               netcdf_store%heat_flux_sensible(store_xpos,store_ypos,&
               store_tpos)
          surface%significant_wave_height2 = &
               netcdf_store%significant_wave_height(store_xpos,store_ypos,&
               store_tpos)
          surface%wave_period2 = netcdf_store%wave_period(store_xpos,&
               store_ypos,store_tpos)
          surface%wave_dir2 = netcdf_store%wave_dir(store_xpos,&
               store_ypos,store_tpos)
          surface%model_sst2 = netcdf_store%model_sst(store_xpos,&
               store_ypos,store_tpos)
          surface%temp_atmos2 = netcdf_store%temp_atmos(store_xpos,&
               store_ypos,store_tpos)
          surface%specific_humidity_atmos2 = &
               netcdf_store%specific_humidity_atmos(store_xpos,&
               store_ypos,store_tpos)
          surface%pressure2 = &
               netcdf_store%pressure(store_xpos,store_ypos,store_tpos)

       ENDIF

       !
       ! Get time - note time is in days from start of year
       !
       newtime = surface%Time1 + (istep-0.5_Dble)*params%dt1 / 86400.0_Dble

       !
       ! Set time
       !
       surface%start_time = surface%time1
       surface%time = surface%start_Time

       !
       ! Define clear sky fraction for both times - if solar flux large enough
       ! Note that index = 0. for clear sky, 1. for cloudy
       !      
       IF( ABS(surface%downwelling_solar1) .ge. min_solar )THEN
          CALL Solar_Properties(surface%time1,surface%latitude,&
               surface%longitude,-surface%downwelling_solar1,sat_za,&
               surface%clear_sky_index1,tot_solar_clear_sky)
       ELSE
          surface%clear_sky_index1 = 1.0_RealType
       ENDIF

       IF( ABS(surface%downwelling_solar2) .ge. min_solar )THEN
          CALL Solar_Properties(surface%time2,surface%latitude,&
               surface%longitude,-surface%downwelling_solar2,sat_za,&
               surface%clear_sky_index2,tot_solar_clear_sky)
       ELSE
          surface%clear_sky_index2 = 1.0_RealType
       ENDIF

       surface%fluxes_init = .TRUE.
    ELSE

       IF( .not. surface%fluxes_init )THEN
          CALL out_ERROR('Fluxes not initialized','Get_Fluxes','get_data.f90')
       ENDIF

       ! 
       ! For all other cases - work out if have crossed to next NWP data
       !
       ! newtime is at 1/2 time step
       !
       newtime = surface%start_Time + (istep-0.5_Dble)*params%dt1 / &
            86400.0_Dble
       surface%time = surface%start_Time + (istep-1)*params%dt1 / &
            86400.0_Dble
       IF( newtime .gt. surface%time2 )THEN

          !
          ! Move time2 to time1 and read in new time2 values
          !
          surface%time1 = surface%time2
          surface%wind_stress_x1 = surface%wind_stress_x2
          surface%wind_stress_y1 = surface%wind_stress_y2
          surface%downwelling_solar1 = surface%downwelling_solar2
          surface%heat_flux_longwave1 = surface%heat_flux_longwave2
          surface%heat_flux_latent1 = surface%heat_flux_latent2
          surface%heat_flux_sensible1 = surface%heat_flux_sensible2
          surface%significant_wave_height1 = surface%significant_wave_height2
          surface%wave_period1 = surface%wave_period2
          surface%wave_dir1 = surface%wave_dir2
          surface%model_sst1 = surface%model_sst2
          surface%temp_atmos1 = surface%temp_atmos2
          surface%specific_humidity_atmos1 = surface%specific_humidity_atmos2
          surface%clear_sky_index1 = surface%clear_sky_index2 
          surface%pressure2 = surface%pressure1

          !
          ! Read in next time step
          !
          IF( params%use_ascii_input )THEN
             READ(Unit_Fluxes,*,IOSTAT=IOS)surface%time2,&
                  surface%wind_stress_x2,surface%wind_stress_y2,&
                  surface%downwelling_solar2,surface%heat_flux_longwave2,&
                  surface%heat_flux_latent2,surface%heat_flux_sensible2,&
                  surface%significant_wave_height2,surface%wave_period2,&
                  surface%wave_dir2,surface%model_sst2,&
                  surface%temp_atmos2,surface%specific_humidity_atmos2
             IF( 0 .ne. IOS )THEN
                CALL out_ERROR('End of file Fluxes file','Get_Fluxes',&
                     'get_data.f90')
             ENDIF
             !
             ! For the minute have surface pressure as 1008.
             !
             surface%pressure2 = 1008.0_RealType
          ELSE

             store_tpos = store_tpos + 1
             IF( store_tpos .gt. netcdf_store%ntime )THEN
                CALL out_ERROR('End of model data (netCDF)',&
                     'Get_Fluxes','get_data.f90')
             ENDIF
             
             new_year = INT(netcdf_store%frac_day(store_tpos)/1000.0_Dble)
             IF( new_year .eq. surface%year + 1)THEN
                ! Crossed a year boundary
                IF( isLeap(surface%year) )THEN
                   surface%time2 = 366.0_Dble + &
                        netcdf_store%frac_day(store_tpos) - &
                        surface%year*1000.0_Dble
                ELSE
                   surface%time2 = 365.0_Dble + &
                        netcdf_store%frac_day(store_tpos) - &
                        surface%year*1000.0_Dble
                ENDIF
             ELSE
                surface%time2 = netcdf_store%frac_day(store_tpos) - &
                     new_year*1000.0_Dble
             ENDIF
             surface%wind_stress_x2 = netcdf_store%wind_stress_x(store_xpos,&
                  store_ypos,store_tpos)
             surface%wind_stress_y2 = netcdf_store%wind_stress_y(store_xpos,&
                  store_ypos,store_tpos)
             !
             ! Note downwelling solar has to be negative but is +ve in NetCDF
             ! file
             !
             surface%downwelling_solar2 = &
                  -netcdf_store%downwelling_solar(store_xpos,store_ypos,&
                  store_tpos)
             surface%heat_flux_longwave2 = &
                  netcdf_store%heat_flux_longwave(store_xpos,store_ypos,&
                  store_tpos)
             surface%heat_flux_latent2 = &
                  netcdf_store%heat_flux_latent(store_xpos,store_ypos,&
                  store_tpos)
             surface%heat_flux_sensible2 = &
                  netcdf_store%heat_flux_sensible(store_xpos,store_ypos,&
                  store_tpos)
             surface%significant_wave_height2 = &
                  netcdf_store%significant_wave_height(store_xpos,store_ypos,&
                  store_tpos)
             surface%wave_period2 = netcdf_store%wave_period(store_xpos,&
                  store_ypos,store_tpos)
             surface%wave_dir2 = netcdf_store%wave_dir(store_xpos,&
                  store_ypos,store_tpos)
             surface%model_sst2 = netcdf_store%model_sst(store_xpos,&
                  store_ypos,store_tpos)
             surface%temp_atmos2 = netcdf_store%temp_atmos(store_xpos,&
                  store_ypos,store_tpos)
             surface%specific_humidity_atmos2 = &
                  netcdf_store%specific_humidity_atmos(store_xpos,&
                  store_ypos,store_tpos)
             surface%pressure2 = &
                  netcdf_store%pressure(store_xpos,store_ypos,store_tpos)
          ENDIF

          !
          ! Get new clear sky index
          !          
          IF( ABS(surface%downwelling_solar2) .ge. min_solar )THEN
             CALL Solar_Properties(surface%time2,surface%latitude,&
                  surface%longitude,-surface%downwelling_solar2,sat_za,&
                  surface%clear_sky_index2,tot_solar_clear_sky)
          ELSE
             surface%clear_sky_index2 = 1.0_RealType
          ENDIF

       ENDIF

       IF( newtime .gt. surface%time2 )THEN
          CALL out_ERROR('time2 < current time','Get_Fluxes','get_data.f90')
       ENDIF

    ENDIF

    !
    ! Get time fraction between time1 to time2
    !
    fraction = REAL((newtime - surface%time1)/(surface%time2-surface%time1),&
         KIND=RealType)

    !
    ! Interpolate all but solar intensity to current time
    !
    surface%wind_stress_x = (1._RealType-fraction)*surface%wind_stress_x1 + &
         fraction*surface%wind_stress_x2
    surface%wind_stress_y = (1._RealType-fraction)*surface%wind_stress_y1 + &
         fraction*surface%wind_stress_y2
    surface%downwelling_solar = (1._RealType-fraction)*surface%downwelling_solar1 + &
         fraction*surface%downwelling_solar2
    surface%heat_flux_longwave = (1._RealType-fraction) * &
         surface%heat_flux_longwave1 + &
         fraction*surface%heat_flux_longwave2
    surface%heat_flux_latent = (1._RealType-fraction) * &
         surface%heat_flux_latent1 + &
         fraction*surface%heat_flux_latent2
    surface%heat_flux_sensible = (1._RealType-fraction) * &
         surface%heat_flux_sensible1 + &
         fraction*surface%heat_flux_sensible2
    surface%significant_wave_height = (1._RealType-fraction) * &
         surface%significant_wave_height1 + &
         fraction*surface%significant_wave_height2
    surface%wave_period = (1._RealType-fraction) * &
         surface%wave_period1 + fraction*surface%wave_period2
    surface%wave_dir = (1._RealType-fraction) * &
         surface%wave_dir1 + fraction*surface%wave_dir2
    surface%model_sst = (1._RealType-fraction) * &
         surface%model_sst1 + fraction*surface%model_sst2
    surface%temp_atmos = (1._RealType-fraction) * &
         surface%temp_atmos1 + fraction*surface%temp_atmos2
    surface%specific_humidity_atmos = (1._RealType-fraction) * &
         surface%specific_humidity_atmos1 + &
         fraction*surface%specific_humidity_atmos2
    surface%pressure = (1._RealType-fraction)*surface%pressure1 + &
         fraction*surface%pressure2

    surface%heat_flux_longwave = (1._RealType-fraction) * &
         surface%heat_flux_longwave1 + &
         fraction*surface%heat_flux_longwave2


    !
    ! If langmuir circulation/stokes drift requested, check to see if wave 
    ! information exists
    !
    IF( params%use_langmuir_param )THEN
       !
       ! If information doesn't exist it should be set to -99 or -9999
       ! depending on the input format
       !
       IF( surface%significant_wave_height1 .lt. 0._RealType .or. &
            surface%significant_wave_height2 .lt. 0._RealType )THEN
          ! No wave information - ignore langmuir circ/stokes drift for this 
          ! position (unless estimated values to be used)
          !
          params%use_langmuir = .FALSE.
       ELSE
          !
          ! Some versions of the NetCDF file have NaN in - so check
          !
          ! Note that a NaN doesn't even equal itself...
          !
          IF( surface%significant_wave_height1 .ne. &
               surface%significant_wave_height1 .or. &
               surface%significant_wave_height2 .ne. &
               surface%significant_wave_height2 )THEN
             params%use_langmuir = .FALSE.
          ELSE
             !
             ! Wave information exists - do langmuir circ/stokes drift
             !
             params%use_langmuir = .TRUE.
          ENDIF
       ENDIF
    ENDIF

    !
    ! For solar, interpolation will be based on the clear sky index.
    ! Since the values are unreliable at nighttime, we use a different logic
    ! based on which of the bracketing observations are daytime.
    !
    
    !
    ! If nighttime for both
    !
    IF( ABS(surface%downwelling_solar1) .lt. min_solar .and. &
         ABS(surface%downwelling_solar2) .lt. min_solar )THEN
       !
       ! solar radiation == 0.0
       !
       surface%qsol = 0.0_RealType
    !
    ! If nighttime for second
    !
    ELSE IF( ABS(surface%downwelling_solar1) .ge. min_solar .and. &
         ABS(surface%downwelling_solar2) .lt. min_solar )THEN
       cli = surface%clear_sky_index1
       !
       ! Get clear sky solar
       !
       CALL Solar_Properties(newtime,surface%latitude,surface%longitude,&
            0.0_RealType,sat_za,cli_dummy,total_solar)
       surface%qsol = -((1.0_RealType-cli) * total_solar)
    !
    ! If nighttime for first
    !
    ELSE IF( ABS(surface%downwelling_solar1) .lt. min_solar .and. &
         ABS(surface%downwelling_solar2) .ge. min_solar )THEN
       cli = surface%clear_sky_index2
       !
       ! Get clear sky solar
       !
       CALL Solar_Properties(newtime,surface%latitude,surface%longitude,&
            0.0_RealType,sat_za,cli_dummy,total_solar)
       surface%qsol = -((1.0_RealType-cli) * total_solar)
    !
    ! If daytime for both
    !
    ELSE IF( ABS(surface%downwelling_solar1) .ge. min_solar .and. &
         ABS(surface%downwelling_solar2) .ge. min_solar )THEN
       cli = (1.-fraction)*surface%clear_sky_index1 + &
            fraction*surface%clear_sky_index2
       !
       ! Get clear sky solar
       !
       CALL Solar_Properties(newtime,surface%latitude,surface%longitude,&
            0.0_RealType,sat_za,cli_dummy,total_solar)
       surface%qsol = -((1.0_RealType-cli) * total_solar)
    ENDIF

    !
    ! If there was an error, set solar to 0.0
    !
    IF( surface%qsol .lt. -solar_constant )THEN
       surface%qsol = 0.0_RealType
    ENDIF

    !
    ! Calculate some other surface parameters
    !
    TauTot = SQRT(surface%wind_stress_x**2+surface%wind_stress_y**2)
    surface%Ustar_atmos = SQRT(TauTot/rho_atmos)
    surface%wind_speed = surface%Ustar_atmos/sqrt_cd
    surface%WU_surf = -surface%wind_stress_x/rho_ocean
    surface%WV_surf = -surface%wind_stress_y/rho_ocean

    !
    ! Get Ustar = Ustar_atmos*sqrt(rho_air/rho_sea)
    !
    surface%Ustar = surface%Ustar_atmos*sqrt_rho_air_sea

    !
    ! Get net longwave and solar flux at surface
    !
    total_heat = surface%heat_flux_longwave + &
         surface%heat_flux_latent + surface%heat_flux_sensible
    surface%net_longwave = total_heat * 0.246e-6_RealType
    surface%net_solar = surface%qsol * 0.246e-6_RealType

    !
    ! If required modify the fluxes taking into account diurnal warming
    ! if any
    !
    IF( params%modify_fluxes )THEN
       !
       ! Modify fluxes on the basis of diurnal warming
       !
       CALL Modify_Fluxes( surface, profile )
    END IF

    IF( params%use_langmuir_param )THEN
       IF( params%use_langmuir )THEN
          CALL Get_Stokes( surface, profile )
       ELSE IF( params%use_estimated_wavedata )THEN
          CALL Get_Estimated_Stokes( surface, profile )
       ENDIF
    ENDIF

    !
    ! breakwave turbulent scale
    !
    ! Note in the case of no wave data, uses estimated value
    !
    surface%breakwave_turb_scale = 1.6_RealType*surface%significant_wave_height
    
    !
    ! If first run set tskin to model sst at this point 
    ! With be modified as the code runs
    !
    IF( 1 .eq. istep )THEN
       surface%tskin = surface%model_sst
    ENDIf

    !
    ! Set rain parameters - defaulted to zero
    !
    IF( params%use_evaporation )THEN
       surface%evaporation = surface%heat_flux_latent / &
            ((2.5008E-6_RealType - 2.3E3_RealType * surface%tskin)*rho_ocean)
    ELSE
       surface%evaporation = 0.0_RealType
    ENDIF
    surface%precipitation = 0.0_RealType

    !
    ! Modify salinity flux based on rainfall
    !
    surface%salinity_flux = -profile%salinity(1)*(surface%evaporation-&
         surface%precipitation)

    !
    ! Set read in flux files
    !
    surface%fluxes_init = .TRUE.
    surface%surface_there = .TRUE.

  END SUBROUTINE Get_Fluxes

  !=================================================================
  !
  ! NAME:
  !
  !      Modify_Fluxes
  !
  ! FUNCTION:
  !
  !      Modify surface fluxes based on warming
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Modifies the surface fluxes based on changes in the surface 
  !      temperature caused by Diurnal warming
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Modify_Fluxes(surface,profile)
  !
  ! INPUTS:
  !
  !      profile   - input profile
  !
  ! OUTPUTS:
  !
  ! INPUTS/OUTPUTS:
  !
  !      surface   - structure with surface/model parameters
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Based on code supplied by Gary Wick NOAA/OAR
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !      
  !
  SUBROUTINE Modify_Fluxes( surface, profile )

    TYPE(Surface_type), INTENT(INOUT) :: surface
    TYPE(Profile_type), INTENT(IN) :: profile

    ! Local variables
    REAL(RealType) :: diurnal_warming
    REAL(RealType) :: new_model_sst
    REAL(RealType) :: correction
    REAL(RealType) :: new_diff    
    REAL(RealType) :: old_diff    
    REAL(RealType) :: new_specific_humidity
    REAL(RealType) :: model_specific_humidity
    REAL(RealType) :: new_flux 
    REAL(RealType) :: old_flux
    REAL(RealType) :: diff_flux

    !
    ! Get diurnal warming from current profile
    !
    diurnal_warming = profile%Temp(1)-profile%Temp(profile%pos_reference)
    new_model_sst = surface%model_sst + diurnal_warming

    !
    ! Correct sensible heat flux for DW
    !
    old_diff = surface%model_sst-surface%temp_atmos
    IF( ABS(old_diff) .gt. 0.01_RealType )THEN
       new_diff = new_model_sst-surface%temp_atmos
       correction = new_diff/old_diff
       surface%heat_flux_sensible = surface%heat_flux_sensible * correction
       IF( ABS(surface%heat_flux_sensible) .gt. 10._RealType .and. &
            ((new_diff .gt. 0._RealType .and. surface%heat_flux_sensible .lt. &
            0._RealType) .or. (new_diff .lt. 0._RealType .and. &
            surface%heat_flux_sensible .gt. 0._RealType)) )THEN
          surface%heat_flux_sensible = -surface%heat_flux_sensible
       ENDIF
    ENDIF

    !
    ! Correct latent heat flux for DW
    !
    ! Must first compute saturation specific humidities based on surface
    ! temperatures
    !
    CALL humidity(new_model_sst,surface%pressure,new_specific_humidity)
    CALL humidity(surface%model_sst,surface%pressure,model_specific_humidity)

    old_diff = model_specific_humidity-surface%specific_humidity_atmos
    IF( ABS(old_diff) .gt. 0.01_RealType )THEN
       surface%heat_flux_latent = surface%heat_flux_latent * &
            (new_diff / old_diff)
       IF( ABS(surface%heat_flux_latent) .gt. 10._RealType .and. &
            ((new_diff .gt. 0._RealType .and. &
            surface%heat_flux_latent .lt. 0._RealType) .or. &
            (new_diff .lt. 0._RealType .and. &
            surface%heat_flux_latent .gt. 0._RealType)) )THEN
          surface%heat_flux_latent = -surface%heat_flux_latent
       ENDIF
    ENDIF

    !
    ! Longwave heat flux
    !
    ! Note sst's assumed degree C so convert to K for Stefans-Boltzmann law
    !
    new_flux = 5.67E-8_RealType*(new_model_sst+convert_to_K)**4
    old_flux = 5.67E-8_RealType*(surface%model_sst+convert_to_K)**4

    diff_flux = old_flux - surface%heat_flux_longwave/0.97_RealType
    
    surface%heat_flux_longwave = 0.97*(new_flux - diff_flux)

  END SUBROUTINE Modify_Fluxes

  !=================================================================
  !
  ! NAME:
  !
  !      humidity
  !
  ! FUNCTION:
  !
  !      Get the humidity at the surface of the ocean
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Get the specific humidity at the surface of the ocean
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL humidity(sst,pressure,Q)
  !
  ! INPUTS:
  !
  !      sst      - Sea Surface Temperature
  !      pressure - surface pressure
  !
  ! OUTPUTS:
  !
  !      Q        - output specific humidity
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Based on code supplied by Gary Wick NOAA/OAR
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !      
  !
  SUBROUTINE humidity(sst,pressure,Q)

    REAL(RealType), INTENT(IN) :: sst
    REAL(RealType), INTENT(IN) :: pressure
    REAL(RealType), INTENT(OUT) :: Q

    !
    ! humidity in mb
    !
    Q = (1.0007_RealType+3.46E-6_RealType*pressure)*6.1121_RealType * &
         exp(17.502_RealType*sst/(240.97_RealType+sst))
    
    !
    ! Reduce for salinity Kraus 1972 p. 46
    !
    Q = Q*0.98_RealType
    
    ! 
    ! Convert from mb to specific humidity g/kg
    !
    Q = 621.97_RealType*(Q/(pressure-0.378_RealType*Q))

  END SUBROUTINE humidity

  !=================================================================
  !
  ! NAME:
  !
  !      Get_Stokes
  !
  ! FUNCTION:
  !
  !      Routine to estimate the stokes drift
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Comment from original code:
  !      
  !c     Routine to estimate the stokes drift given the significant wave
  !c     height and wave period
  !c
  !c     Approach here for surface stokes drift is following Webb and 
  !c     Fox-Kemper assuming the Pierson and Moskowitz spectrum.
  !c     Assumption of exponential decay with depth is from Janssen
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Get_Stokes(surface,profile)
  !
  ! INPUTS:
  !
  !      surface  - surface structure
  !
  ! OUTPUTS:
  !
  ! INPUTS/OUTPUTS:
  !
  !      profile  - depth profile
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Based on code supplied by Gary Wick NOAA/OAR
  !
  !      Original F90 version by Jonathan P.D. Mittaz 9/10/2013
  !      
  !

  SUBROUTINE Get_Stokes( surface, profile )

    TYPE(surface_type), INTENT(IN) :: surface
    TYPE(profile_type), INTENT(INOUT) :: profile

    ! Local variables
    INTEGER(Int4) :: K
    REAL(RealType) :: US_top
    REAL(RealType) :: depth_scale
    REAL(RealType) :: SU

    !
    ! Make sure we have a windspeed (based on Ustar)
    !
    IF( 0._RealType .lt. surface%wind_speed .and. &
         100._RealType .gt. surface%wind_speed )THEN
       US_top = 4.289_RealType * PI**3 * surface%significant_wave_height**3/&
            (grav*surface%wave_period**3)
       
       depth_scale = grav/(surface%wind_speed**2)
       
       DO K=1,profile%nlevels
          SU = US_Top * exp(2.0_RealType*depth_scale*profile%zz(K) * &
               profile%Bottom)
          profile%Velocity_U_Stokes(K) = SU*sin(surface%wave_dir*d2r)
          profile%Velocity_V_Stokes(K) = SU*cos(surface%wave_dir*d2r)
       END DO
    ENDIF

  END SUBROUTINE Get_Stokes

  !=================================================================
  !
  ! NAME:
  !
  !      Get_Estimated_Stokes
  !
  ! FUNCTION:
  !
  !      Routine to estimate the stokes drift where there is no direct
  !      wave data available.
  !
  !      based on Gary Wicks routine fulldev.f
  !
  ! TYPE/LANGUAGE:
  !
  !      Fortran 90
  !
  ! DESCRIPTION:
  !
  !      Comment from original code:
  !      
  !c      Routine to estimate signficant wave height and Stokes drift based on
  !c      wind speed and the assumption of fully developed seas
  !c
  !c      Values here taken following Webb and Fox-Kemper for the Pierson &
  !c      Moskowitz spectrum assuming fully developed seas
  !
  ! SUBROUTINE/FUNCTION CALLS:
  !
  ! CALLING SEQUENCE:
  !
  !      CALL Get_Estiamted_Stokes(surface,profile)
  !
  ! INPUTS:
  !
  !      surface  - surface structure
  !
  ! OUTPUTS:
  !
  ! INPUTS/OUTPUTS:
  !
  !      profile  - depth profile
  !
  ! RETURNS:
  !
  ! HISTORY
  !
  !      Based on code supplied by Gary Wick NOAA/OAR
  !
  !      Original F90 version by Jonathan P.D. Mittaz 8/01/2014
  !      
  !
  SUBROUTINE Get_Estimated_Stokes( surface, profile )

    TYPE(surface_type), INTENT(IN) :: surface
    TYPE(profile_type), INTENT(INOUT) :: profile

    ! Local variables
    INTEGER(Int4) :: K
    REAL(RealType) :: US_top
    REAL(RealType) :: depth_scale
    REAL(RealType) :: SU

    !
    ! Make sure we have a windspeed (based on Ustar)
    !
    IF( 0._RealType .lt. surface%wind_speed .and. &
         100._RealType .gt. surface%wind_speed )THEN
       US_top = 0.0162*surface%wind_speed
       
       depth_scale = 4.15*grav/(surface%wind_speed**2)
       
       DO K=1,profile%nlevels
          SU = US_Top * exp(2.0_RealType*depth_scale*profile%zz(K) * &
               profile%Bottom)
          profile%Velocity_U_Stokes(K) = SU*sin(surface%wave_dir*d2r)
          profile%Velocity_V_Stokes(K) = SU*cos(surface%wave_dir*d2r)
       END DO
    ENDIF

  END SUBROUTINE Get_Estimated_Stokes

END MODULE Get_Data
